// -*- C++ -*-
/*!
 * @file CorbaNaming.h
 * @brief CORBA naming service helper class
 * @date $Date: 2007-12-31 03:08:02 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: CorbaNaming.h 3185 2018-01-23 00:38:28Z miyamoto $
 *
 */

#ifndef RTC_CORBANAMING_H
#define RTC_CORBANAMING_H

#include <rtm/RTC.h>

// STL includes
#include <map>
#include <string>
#include <vector>

/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RTR|[lg
 *
 * @else
 *
 * @namespace RTC
 *
 * @brief RT-Component
 *
 * @endif
 */

#ifdef WIN32
#pragma warning( disable : 4290 )
#endif

namespace RTC
{
  /*!
   * @if jp
   * @class CorbaNaming
   * @brief CORBA Naming Service wp[NX
   *
   * ̃NX́ACosNaming::NamingContext ɑ΂郉bp[NXłB
   * CosNaming::NamingContext Iy[VƂقړ@\
   * Iy[V񋟂ƂƂɁAl[R|[lg CosNaming::Name
   * ̑ɕɂ閼O\󂯕tIy[V񋟂B
   *
   * IuWFNg͐A邢͐ CORBA l[T[oɐڑ
   * ȌÃl[T[õ[gReLXgɑ΂ĎX̃Iy[V
   * B
   * [Kw̃l[~OReLXg̍쐬IuWFNg̃oChɂāA
   * r̃ReLXg݂ȂꍇłAIɃReLXgoCh
   * ړĨReLXgIuWFNg̃oChsƂłB
   *
   * @since 0.4.0
   *
   * @else
   * @class CorbaNaming
   * @brief CORBA Naming Service helper class
   *
   * This class is a wrapper class of CosNaming::NamingContext.
   * Almost the same operations which CosNaming::NamingContext has are
   * provided, and some operation allows string naming representation of
   * context and object instead of CosNaming::Name.
   *
   * The object of the class would connect to a CORBA naming server at
   * the instantiation or immediately after instantiation.
   * After that the object invokes operations to the root context of it.
   * This class realizes forced binding to deep NamingContext, without binding
   * intermediate NamingContexts explicitly.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class CorbaNaming
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * @param orb ORB
     *
     * @else
     *
     * @brief Consructor
     *
     * @param orb ORB
     *
     * @endif
     */
    CorbaNaming(CORBA::ORB_ptr orb);
    
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * @param orb ORB
     * @param name_server l[T[o̖
     *
     * @else
     *
     * @brief Consructor
     *
     * @param orb ORB
     * @param name_server Name of the name server
     *
     * @endif
     */
    CorbaNaming(CORBA::ORB_ptr orb, const char* name_server);
    
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * @else
     * 
     * @brief Virtual destructor
     * 
     * @endif
     */
    virtual ~CorbaNaming(void){};
    
    /*!
     * @if jp
     *
     * @brief l[~OT[rX̏
     * 
     * w肳ꂽl[T[õl[~OT[rX܂B
     * 
     * @param name_server l[T[o̖
     * 
     * @else
     *
     * @brief Initialize the Naming Service
     * 
     * Initialize the Naming Service on the specified name server.
     * 
     * @param name_server Name of the name server
     * 
     * @endif
     */
    void init(const char* name_server);

    bool isAlive();
    
    typedef CORBA::SystemException SystemException;
    typedef CosNaming::NamingContext::NotFound      NotFound;
    typedef CosNaming::NamingContext::CannotProceed CannotProceed;
    typedef CosNaming::NamingContext::InvalidName   InvalidName;
    typedef CosNaming::NamingContext::AlreadyBound  AlreadyBound;
    typedef CosNaming::NamingContext::NotEmpty      NotEmpty;
    typedef CosNaming::NamingContextExt::InvalidAddress InvalidAddress;
    typedef std::vector<CORBA::Object_ptr> ObjectList;
    
    /*!
     * @if jp
     *
     * @brief Object  bind 
     *
     * CosNaming::bind() Ƃقړ̓邪Aɗ^ꂽl[T[o
     * [gReLXgɑ΂bind()Ăяo_قȂB
     *
     * Name <name>  Object <obj> 𓖊Y NamingContext ɃoChB
     * c_n  n Ԗڂ NameComponent 킷ƂƁA
     * name  n  NameComponent 琬ƂAȉ̂悤ɈB
     *
     * cxt->bind(<c_1, c_2, ... c_n>, obj) ͈ȉ̑ƓłB
     * cxt->resolve(<c_1, ... c_(n-1)>)->bind(<c_n>, obj)
     *
     * Ȃ킿A1Ԗڂn-1Ԗڂ̃ReLXgAn-1Ԗڂ̃ReLXg
     *  name <n> Ƃā@obj  bind B
     * OɎQ <c_1, ... c_(n-1)>  NemingContext ́A
     * bindContext()  rebindContext() ŊɃoChς݂łȂ΂ȂȂB
     *  <c_1, ... c_(n-1)>  NamingContext ݂Ȃꍇɂ́A
     * NotFound OB
     *
     * AoChtO force  true ̎́A<c_1, ... c_(n-1)>
     * ݂ȂꍇɂAċAIɃReLXgoChȂA
     * ŏII obj 𖼑O name <c_n> ɃoChB
     *
     * ̏ꍇłAn-1Ԗڂ̃ReLXg name<n> ̃IuWFNg
     * (Object 邢 ReLXg) oChĂ
     * AlreadyBound OB
     *
     * @param name IuWFNgɕt閼O NameComponent
     * @param obj ֘At Object
     * @param force truȅꍇAr̃ReLXgIɃoCh
     *              (ftHgl:true)
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     * @exception AlreadyBound name <c_n>  Object łɃoChĂB
     *
     * @else
     *
     * @brief Bind object on specified name component position
     *
     * Almost the same operation as CosNaming::bind(), but there is a difference 
     * that bind() is invoked for the root context of the given name server.
     *
     * Bind between Name <name> and Object <obj> on this NamingContext.
     * If c_n indicates the n-th of NameComponent,
     * when name consists of n pieces of NameComponent, it is handled as 
     * follows. 
     *
     * cxt->bind(<c_1, c_2, ... c_n>, obj) is the same as the following
     * operation.
     * cxt->resolve(<c_1, ... c_(n-1)>)->bind(<c_n>, obj)
     *
     * In other word, resolve from the first to the (n-1)th context and bind 
     * obj as name<n> on the (n-1)th context.
     * NemingContext of <c_1, ... c_(n-1)> for resolving name must be already 
     * bound in bindContext() or rebindContext().
     * If NamingContext of <c_1, ... c_(n-1)> does not exist, NotFound excption
     * will occur.
     *
     * However, when flag of forced bind is true, even if <c_1, ... c_(n-1)> does
     * not exist, finally obj will be bound to name name <c_n> by binding to 
     * the context recursively.
     *
     * Even in any case, if the object of name<n> (Object or context) is bound
     * on the (n-1)th context, AlreadyBound exception will occur.
     *
     * @param name NameComponent of name applied to object
     * @param obj Object that is associated
     * @param force If true, the intermediate context is bound forcibly.
     *              (The default value:true)
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     * @exception AlreadyBound The object of name<c_n> is already bound.
     *
     * @endif
     */
    void bind(const CosNaming::Name& name, CORBA::Object_ptr obj,
	      const bool force = 1)
      throw (SystemException, NotFound, CannotProceed,
             InvalidName, AlreadyBound);
    
    /*!
     * @if jp
     *
     * @brief Object  bind 
     *
     * Object  bind ۂɗ^閼O\ł邱ƈȊÓAbind()
     * ƓłBbind(toName(string_name), obj) ƓB
     *
     * @param string_name IuWFNgɕt閼O̕\
     * @param obj ֘AtIuWFNg
     * @param force truȅꍇAr̃ReLXgIɃoCh
     *              (ftHgl:true)
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     * @exception AlreadyBound name <n>  Object łɃoChĂB
     *
     * @else
     *
     * @brief Bind object on specified string name position
     *
     * This is the same as bind() except as the given name is string 
     * representation when Object is bound. 
     * bind(toName(string_name),obj) is the same. 
     *
     * @param string_name The string representation of name applied to object
     * @param obj Object that is associated
     * @param force If true, the intermediate context is bound forcibly.
     *              (The default value:true)
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     * @exception AlreadyBound The object of name<c_n> is already bound.
     *
     * @endif
     */
    void bindByString(const char* string_name, CORBA::Object_ptr obj,
		      const bool force = 1)
      throw (SystemException, NotFound, CannotProceed,
             InvalidName, AlreadyBound);
    
    /*!
     * @if jp
     *
     * @brief r̃ReLXgċAI bind Ȃ Object  bind 
     *
     * context ŗ^ꂽ NamingContext ɑ΂āAname Ŏw肳ꂽ
     * l[R|[lg <c_1, ... c_(n-1)>  NamingContext Ƃ
     * ȂAO <c_n> ɑ΂ obj  bind B
     * A<c_1, ... c_(n-1)> ɑΉ NamingContext Ȃꍇɂ
     * V NamingContext oChB
     *
     * ŏII <c_1, c_2, ..., c_(n-1)> ɑΉ NamingContext 
     * ܂͉ꂽŁACosNaming::bind(<c_n>, object) ĂяoB
     * ̂ƂAłɃoCfBO݂ AlreadyBoundOB
     *
     * r̃ReLXgߒŁA悤ƂReLXg
     * O NamingContext ł͂Ȃ Binding ݂ꍇA
     * CannotProceed O𒆎~B
     *
     * @param context bind Jn@NamingContext
     * @param name IuWFNgɕt閼Õl[R|[lg
     * @param obj ֘AtIuWFNg
     *
     * @exception CannotProceed <c_1, ..., c_(n-1)> ɑΉ NamingContext 
     *            ̂ЂƂAł NamingContext ȊO object ɃoCh
     *            ĂApłȂB
     * @exception InvalidName O name s
     * @exception AlreadyBound name <c_n> ɂłɉ炩 object oCh
     *            ĂB
     * @else
     *
     * @brief Bind intermediate context recursively and bind object
     *
     * For NamingContext given in context, bind obj to name <c_n> with solving
     * name component <c_1, ... c_(n-1)> specified by name as NamingContext.
     * Bind new NamingContext when there is no NamingContext corresponding to
     * c_(n-1) >.
     *
     * Finally, NamingContext corresponding to <c_1, c_2, ..., c_(n-1)> 
     * will be generated, or CosNaming::bind(<c_n>, object) will be invoked
     * after solving. At this time, if the binding already exists, 
     * the AlreadyBound exception will occur.
     *
     * During process, when Binding that is not NamingContext of the same name
     * as the context for solving exists, CannotProceed exception will occur
     * and stop processing.
     *
     * @param context NamingContext that starts the bind
     * @param name NameComponent of name applied to object
     * @param obj Object that is associated
     *
     * @exception CannotProceed Since one of NamingContext corresponding to
     *                          <c_1, ..., c_(n-1)> is already bound to object
     *                          other than NamingContext and processing cannot 
     *                          be continued
     * @exception InvalidName name 'name' is invalid.
     * @exception AlreadyBound The object of name<c_n> is already bound.
     *
     * @endif
     */
    void bindRecursive(CosNaming::NamingContext_ptr context,
		       const CosNaming::Name& name,
		       CORBA::Object_ptr obj)
      throw (SystemException, CannotProceed, InvalidName, AlreadyBound);
    
    /*!
     * @if jp
     *
     * @brief Object  rebind 
     *
     * name Ŏw肳ꂽ Binding łɑ݂ꍇ bind() Ɠ
     * łBoCfBOłɑ݂ꍇɂ́AVoCfBO
     * uB
     *
     * @param name IuWFNgɕt閼O NameComponent
     * @param obj ֘AtIuWFNg
     * @param force truȅꍇAr̃ReLXgIɃoCh
     *              (ftHgl:true)
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName O name s
     *
     * @else
     *
     * @brief Rebind object
     *
     * This is the same as bind() except as Binding specified by name 
     * already exists. If the binding already exists, new binding will be 
     * replaced.
     *
     * @param name NameComponent of name applied to object
     * @param obj Object that is associated
     * @param force If true, the intermediate context is bound forcibly.
     *              (The default value:true)
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName Name 'name' is invalid.
     *
     * @endif
     */
    void rebind(const CosNaming::Name& name, CORBA::Object_ptr obj,
		const bool force = 1)
      throw (SystemException, NotFound, CannotProceed, InvalidName);
    
    /*!
     * @if jp
     *
     * @brief Object  rebind 
     *
     * Object  rebind ۂɗ^閼O\ł邱ƈȊO rebind()
     * ƓłBrebind(toName(string_name), obj) ƓB
     *
     * @param string_name IuWFNgɕt閼O̕\
     * @param obj ֘AtIuWFNg
     * @param force truȅꍇAr̃ReLXgIɃoCh
     *              (ftHgl:true)
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     *
     * @else
     *
     * @brief Rebind Object
     *
     * This is the same as rebind() except as the given name is string
     * representation when object is rebound. rebind(toName(string_name), obj) 
     * is the same.
     *
     * @param string_name NameComponent of name applied to object
     * @param obj Object that is associated
     * @param force If true, the intermediate context is bound forcibly.
     *              (The default value:true)
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     *
     * @endif
     */
    void rebindByString(const char* string_name, CORBA::Object_ptr obj,
			const bool force = 1)
      throw (SystemException, NotFound, CannotProceed, InvalidName);
    
    /*!
     * @if jp
     *
     * @brief r̃ReLXg bind Ȃ Object  rebind 
     *
     * name <c_n> Ŏw肳ꂽ NamingContext  Object łɑ݂
     * ꍇ bindRecursive() ƓłB
     *
     * name <c_n> Ŏw肳ꂽoCfBOłɑ݂ꍇɂ́A
     * VoCfBOɒuB
     *
     * @param context bind Jn@NamingContext
     * @param name IuWFNgɕt閼O NameComponent
     * @param obj ֘AtIuWFNg
     *
     * @exception CannotProceed r̃ReLXgłȂB
     * @exception InvalidName ^ꂽ name sB
     *
     * @else
     *
     * @brief Bind intermediate context recursively and rebind object
     *
     * This is the same as bindRecursive() except as NamingContext 
     * or Object specified by name <c_n> already exists.
     *
     * If the binding specified by name <c_n> already exists, 
     * new binding will be replaced.
     *
     * @param context NamingContext that starts the bind
     * @param name NameComponent of name applied to object
     * @param obj Object that is associated
     *
     * @exception CannotProceed The intermediate context cannot resolved.
     * @exception InvalidName The given name is invalid.
     *
     * @endif
     */
    void rebindRecursive(CosNaming::NamingContext_ptr context,
			 const CosNaming::Name& name,
			 CORBA::Object_ptr obj)
      throw (SystemException, CannotProceed, InvalidName);
    
    /*!
     * @if jp
     *
     * @brief NamingContext  bind 
     *
     * bind IuWFNg NamingContext ł邱Ƃ bind() 
     * ƓłB
     *
     * @param name IuWFNgɕt閼Õl[R|[lg
     * @param name_cxt ֘At NamingContext
     * @param force truȅꍇAr̃ReLXgIɃoCh
     *              (ftHgl:true)
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     * @exception AlreadyBound name <c_n>  Object łɃoChĂB
     *
     * @else
     *
     * @brief Bind NamingContext
     *
     * This is the same as bind() except as the bound object is NamingContext.
     *
     * @param name NameComponent of name applied to object
     * @param name_cxt Object that is associated
     * @param force If true, the intermediate context is bound forcibly.
     *              (The default value:true)
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     * @exception AlreadyBound The object of name<c_n> is already bound.
     *
     * @endif
     */
    void bindContext(const CosNaming::Name& name,
		     CosNaming::NamingContext_ptr name_cxt,
		     const bool force = 1)
      throw (SystemException, NotFound, CannotProceed,
             InvalidName, AlreadyBound);
    
    /*!
     * @if jp
     *
     * @brief NamingContext  bind 
     *
     * bind IuWFNg NamingContext ł邱Ƃ
     * bindByString() ƓłB
     *
     * @param string_name IuWFNgɕt閼O̕\
     * @param name_cxt ֘At NamingContext
     * @param force truȅꍇAr̃ReLXgIɃoCh
     *              (ftHgl:true)
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     * @exception AlreadyBound name <n>  Object łɃoChĂB
     *
     * @else
     *
     * @brief Bind NamingContext
     *
     * This is the same as bindByString() except as the bound object is
     * NamingContext.
     *
     * @param string_name String representation of name applied to object
     * @param name_cxt NamingContext that is associated
     * @param force If true, the intermediate context is bound forcibly.
     *              (The default value:true)
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     * @exception AlreadyBound The object of name<n> is already bound.
     *
     * @endif
     */
    void bindContext(const char* string_name,
		     CosNaming::NamingContext_ptr name_cxt,
		     const bool force = 1)
      throw (SystemException, NotFound, CannotProceed,
             InvalidName, AlreadyBound);
    
    /*!
     * @if jp
     *
     * @brief r̃ReLXgċAI bind  NamingContext  bind 
     *
     * bind IuWFNg NamingContext ł邱Ƃ
     * bindRecursive() ƓłB
     *
     * @param context bind Jn@NamingContext
     * @param name IuWFNgɕt閼Õl[R|[lg
     * @param name_cxt ֘At NamingContext
     *
     * @else
     *
     * @brief Bind intermediate context recursively and bind NamingContext
     *
     * This is the same as bindRecursive() except as the bound object
     * is NamingContext.
     *
     * @param context NamingContext that starts the bind
     * @param name NameComponent of name applied to object
     * @param name_cxt NamingContext that is associated
     *
     * @endif
     */
    void bindContextRecursive(CosNaming::NamingContext_ptr context,
			      const CosNaming::Name& name,
			      CosNaming::NamingContext_ptr name_cxt);
    
    /*!
     * @if jp
     *
     * @brief NamingContext  rebind 
     *
     * name Ŏw肳ꂽReLXgłɑ݂ꍇ bindContext() 
     * ƓłB
     * oCfBOłɑ݂ꍇɂ́AVoCfBO
     * uB
     *
     * @param name IuWFNgɕt閼Õl[R|[lg
     * @param name_cxt ֘At NamingContext
     * @param force truȅꍇAr̃ReLXgIɃoCh
     *              (ftHgl:true)
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     *
     * @else
     *
     * @brief Rebind NamingContext
     *
     * This is the same as bindContext() except as context specified
     * by name already exists.
     * If the binding already exists, new binding will be replaced.
     *
     * @param name NameComponent applied to object
     * @param name_cxt Object that is associated
     * @param force If true, the intermediate context is bound forcibly.
     *              (The default value:true)
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName the argument 'name' is invalid.
     *
     * @endif
     */
    void rebindContext(const CosNaming::Name& name,
		       CosNaming::NamingContext_ptr name_cxt,
		       const bool force = 1)
      throw (SystemException, NotFound, CannotProceed, InvalidName);
    
    /*!
     * @if jp
     *
     * @brief NamingContext  rebind 
     *
     * name Ŏw肳ꂽReLXgłɑ݂ꍇ bindContext() 
     * ƓłB
     * oCfBOłɑ݂ꍇɂ́AVoCfBO
     * uB
     *
     * @param string_name IuWFNgɕt閼O̕\
     * @param name_cxt ֘At NamingContext
     * @param force truȅꍇAr̃ReLXgIɃoCh
     *              (ftHgl:true)
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     *
     * @else
     *
     * @brief Rebind NamingContext
     *
     * This is the same as bindContext() except as context specified
     * by name already exists.
     * If the binding already exists, new binding will be replaced.
     *
     * @param string_name String representation of name applied to object
     * @param name_cxt NamingContext that is associated
     * @param force If true, the intermediate context is bound forcibly.
     *              (The default value:true)
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     *
     * @endif
     */
    void rebindContext(const char* string_name,
		       CosNaming::NamingContext_ptr name_cxt,
		       const bool force = 1)
      throw (SystemException, NotFound, CannotProceed, InvalidName);
    
    /*!
     * @if jp
     *
     * @brief r̃ReLXgċAI rebind  NamingContext  rebind 
     *
     * bind IuWFNg NamingContext ł邱Ƃ
     * rebindRecursive() ƓłB
     *
     * @param context bind Jn@NamingContext
     * @param name IuWFNgɕt閼O NameComponent
     * @param name_cxt ֘At NamingContext
     *
     * @else
     *
     * @brief Rebind intermediate context recursively and rebind NamingContext
     *
     * This is the same as rebindRecursive() except as the bound object is 
     * NamingContext.
     *
     * @param context NamingContext that starts the bind
     * @param name NameComponent applied to object
     * @param name_cxt NamingContext that is associated
     *
     * @endif
     */
    void rebindContextRecursive(CosNaming::NamingContext_ptr context,
				const CosNaming::Name& name,
				CosNaming::NamingContext_ptr name_cxt);
    
    /*!
     * @if jp
     *
     * @brief ^ꂽ NameComponent ɃoChĂ Object Ԃ
     *
     * name  bind ĂIuWFNgQƂԂB
     * l[R|[lg <c_1, c_2, ... c_n> ͍ċAIɉB
     * 
     * CosNaming::resolve() Ƃقړ̓邪Aɗ^ꂽ
     * l[T[õ[gReLXgɑ΂ resolve() Ăяo_
     * قȂB
     *
     * @param name ׂIuWFNg̖Õl[R|[lg
     *
     * @return ꂽIuWFNgQ
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     *
     * @else
     *
     * @brief Return object bound on the specified NameComponent
     *
     * Return the object reference that is bound to name.
     * Resolve the name component<c_1, c_2, ... c_n> recursively.
     * 
     * Almost the same operation as CosNaming::resolve(), 
     * but there is a difference that resolve() is invoked for the root context
     * of the given name server.
     *
     * @param name The name component of object name that should be resolved
     *
     * @return The reference to the resolved object
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     *
     * @endif
     */
    CORBA::Object_ptr resolve(const CosNaming::Name& name)
      throw (SystemException, NotFound, CannotProceed, InvalidName);
    
    /*!
     * @if jp
     *
     * @brief ^ꂽ NameComponent ɃoChĂ Object Ԃ
     *
     * name  bind ĂIuWFNgQƂԂB
     * l[R|[lg <c_1, c_2, ... c_n> ͍ċAIɉB
     * 
     * CosNaming::resolve() Ƃقړ̓邪Aɗ^ꂽ
     * l[T[õ[gReLXgɑ΂ resolve() Ăяo_
     * قȂB
     *
     * @param string_name ׂIuWFNg̖O̕\
     *
     * @return ꂽIuWFNgQ
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     *
     * @else
     *
     * @brief Return object bound on the specified name
     *
     * Return the object reference that is bound to name.
     * Resolve the name component<c_1, c_2, ... c_n> recursively.
     * 
     * Almost the same operation as CosNaming::resolve(), 
     * but there is a difference that resolve() is invoked for the root context
     * of the given name server.
     *
     * @param string_name The string representation of object name 
     *             that should be resolved
     *
     * @return The reference to the resolved object
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     *
     * @endif
     */
    CORBA::Object_ptr resolve(const char* string_name)
      throw (SystemException, NotFound, CannotProceed, InvalidName);
    
    /*!
     * @if jp
     *
     * @brief ^ꂽ NameComponent ̃oCfBO폜
     *
     * name  bind ĂIuWFNgQƂԂB
     * l[R|[lg <c_1, c_2, ... c_n> ͍ċAIɉB
     * 
     * CosNaming::unbind() Ƃقړ̓邪Aɗ^ꂽ
     * l[T[õ[gReLXgɑ΂ unbind() Ăяo_
     * قȂB
     *
     * @param name 폜IuWFNg̃l[R|[lg
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     *
     * @else
     *
     * @brief Unbind a binding specified by NameComponent
     *
     * Return the object reference that is bound to name.
     * Resolve the name component<c_1, c_2, ... c_n> recursively.
     * 
     * Almost the same operation as CosNaming::unbind(), 
     * but there is a difference that unbind() is invoked for the root context
     * of the always given name server.
     *
     * @param name The name component of the deleted object
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     *
     * @endif
     */
    void unbind(const CosNaming::Name& name)
      throw (SystemException, NotFound, CannotProceed, InvalidName);
    
    /*!
     * @if jp
     *
     * @brief ^ꂽ NameComponent ̃oCfBO폜
     *
     * name  bind ĂIuWFNgQƂԂB
     * l[R|[lg <c_1, c_2, ... c_n> ͍ċAIɉB
     * 
     * CosNaming::unbind() Ƃقړ̓邪Aɗ^ꂽ
     * l[T[õ[gReLXgɑ΂ unbind() Ăяo_
     * قȂB
     *
     * @param string_name ׂIuWFNg̖O̕\
     *
     * @return ꂽIuWFNgQ
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     *
     * @else
     *
     * @brief Unbind a binding specified by string representation
     *
     * Return the object reference that is bound to name.
     * Resolve the name component<c_1, c_2, ... c_n> recursively.
     * 
     * Almost the same operation as CosNaming::unbind(), 
     * but there is a difference that unbind() is invoked for the root context
     * of the always given name server.
     *
     * @param string_name The string representation of object name
     *                    that should be resolved
     *
     * @return The resolved object reference
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     *
     * @endif
     */
    void unbind(const char* string_name)
      throw (SystemException, NotFound, CannotProceed, InvalidName);
    
    /*!
     * @if jp
     *
     * @brief VReLXg𐶐
     *
     * ^ꂽl[T[oŐꂽ NamingContext ԂB
     * Ԃꂽ NamingContext  bind ĂȂB
     * 
     * @return ꂽV NamingContext
     *
     * @else
     *
     * @brief Create new NamingContext
     *
     * Return NamingContext that has been created on the given name server.
     * The returned NamingContext has not bound yet.
     * 
     * @return New created NamingContext
     *
     * @endif
     */
    CosNaming::NamingContext_ptr newContext();
    
    /*!
     * @if jp
     *
     * @brief VReLXg bind 
     *
     * ^ꂽ name ɑ΂ĐVReLXgoChB
     * ꂽ@NamingContext ̓l[T[oŐꂽ̂łB
     * 
     * @param name NamingContextɕt閼Õl[R|[lg
     * @param force truȅꍇAr̃ReLXgIɃoCh
     *              (ftHgl:true)
     *
     * @return ꂽV NamingContext
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     * @exception AlreadyBound name <n>  Object łɃoChĂB
     *
     * @else
     *
     * @brief Bind new NamingContext
     *
     * Bind new context for the given name.
     * The created NamingContext is a creation on the name server.
     * 
     * @param name NameComponent applied to NamingContext
     * @param force If true, the intermediate context is bound forcibly.
     *              (The default value:true)
     *
     * @return New created NamingContext
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     * @exception AlreadyBound The object of name<n> is already bound.
     *
     * @endif
     */
    CosNaming::NamingContext_ptr
    bindNewContext(const CosNaming::Name& name, bool force = true)
      throw (SystemException, NotFound, CannotProceed,
             InvalidName, AlreadyBound);
    
    /*!
     * @if jp
     *
     * @brief VReLXg bind 
     *
     * ^ꂽɑΉVReLXgoChB
     * ꂽ@NamingContext ̓l[T[oŐꂽ̂łB
     * 
     * @param string_name NamingContextɕt閼O̕\
     * @param force truȅꍇAr̃ReLXgIɃoCh
     *              (ftHgl:true)
     * 
     * @return ꂽV NamingContext
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     * @exception AlreadyBound name <n>  Object łɃoChĂB
     *
     * @else
     *
     * @brief Bind new NamingContext
     *
     * Bind new context corresponding to the given string.
     * The created NamingContext is a creation on the name server.
     * 
     * @param string_name The string representation of name applied to 
     *                    NamingContext
     * @param force If true, the intermediate context is bound forcibly.
     *              (The default value:true)
     * 
     * @return New created NamingContext
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     * @exception AlreadyBound The object of name<n> is already bound.
     *
     * @endif
     */
    CosNaming::NamingContext_ptr
    bindNewContext(const char* string_name, bool force = true)
      throw (SystemException, NotFound, CannotProceed,
             InvalidName, AlreadyBound);
    
    /*!
     * @if jp
     *
     * @brief NamingContext ANeBu
     *
     * context Ŏw肳ꂽ NamingContext ANeBuB
     * context ɑ̃ReLXgoChĂꍇ NotEmpty O
     * B
     * 
     * @param context ANeBu NamingContext
     *
     * @exception NotEmpty Ώcontext ɑ̃ReLXgoChĂB
     *
     * @else
     *
     * @brief Destroy the naming context
     *
     * Destroy the specified naming context.
     * Any bindings should be <unbind> in which the given context is bound to
     * some names before invoking <destroy> operation on it. 
     *
     * @param context NamingContext which is destroied.
     *     
     * @exception NotEmpty The target context is bound to the other context.
     *
     * @endif
     */
    void destroy(CosNaming::NamingContext_ptr context)
      throw (SystemException, NotEmpty);
    
    /*!
     * @if jp
     * @brief NamingContext ċAIɉĔANeBu
     *
     * context ŗ^ꂽ NamingContext ɑ΂āAname Ŏw肳ꂽ
     * l[R|[lg <c_1, ... c_(n-1)>  NamingContext Ƃ
     * ȂAO <c_n> ɑ΂ ANeBusB
     *
     * @param context ANeBu NamingContext
     *
     * @exception NotEmpty Ώcontext ɑ̃ReLXgoChĂB
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     *
     * @else
     *
     * @brief Destroy the naming context recursively
     *
     * For NamingContext given by Context, Destroy name <c_n> with 
     * solving the name component specified by name as NamingContext recursively.
     *
     * @param context NamingContext which is Destroied.
     *
     * @exception NotEmpty The target context is bound to the other context.
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     *
     * @endif
     */
    void destroyRecursive(CosNaming::NamingContext_ptr context)
      throw (SystemException, NotEmpty, NotFound, CannotProceed, InvalidName);
    
    /*!
     * @if jp
     * @brief ׂĂ Binding 폜
     *
     * o^ĂSĂBinding 폜B
     *
     * @else
     * @brief Destroy all bindings
     *
     * Destroy all bindings that are registered.
     *
     * @endif
     */
    void clearAll();
    
    /*!
     * @if jp
     * @brief ^ꂽ NamingContext  Binding 擾
     *
     * w肳ꂽ NamingContext  Binding 擾B
     *
     * @param name_cxt Binding 擾Ώ NamingContext
     * @param how_many Binding 擾Kw̐[
     * @param bl 擾 Binding ێz_
     * @param bi 擾 Binding ǂ邽߂̃Ce[^
     *
     * @else
     * @brief Get Binding of the given NamingContext
     *
     * Get Binding of the given NamingContext.
     *
     * @param name_cxt NamingContext of the getting target Binding
     * @param how_many The depth to get Binding
     * @param bl The holder to hold the got Binding
     * @param bi The iterator to detect the got Binding
     *
     * @endif
     */
    void list(CosNaming::NamingContext_ptr name_cxt,
              CORBA::ULong how_many,
              CosNaming::BindingList_var& bl,
              CosNaming::BindingIterator_var& bi);
    /*!
     * @if jp
     * @brief ^ꂽ Naming pXȉׂ̂ẴoCfBO擾
     *
     * Ŏw肳ꂽl[~O΃pXȉׂ̂ẴoCfBO
     * Ȃ킿oCfBÕ^CvƎQƂ̃Zbg擾BpX́A
     * [gReLXgKw "/"AOƃReLXg "." ŋ
     * ؂łB
     *
     * Ⴆ΁A<name0>.<context0>/.../<name[n]>.<context[n]> w肷
     * <name[n]>.<context[n]> ̉ɃoChĂReLXg
     * ̓IuWFNgt@X̃Xg BindingList ƂĕԂ
     * B
     *
     * @param string_name [in] Ŏw肳^[Qbg̃pX
     * @param bl [out] w肳ꂽpX̉̃oCfBOXg
     *
     * @else
     *
     * @brief Get all the binding under given naming path
     *
     * This operation obtains all the binding list, which are a pair
     * of binding type and reference, under specified absolute naming
     * path. The path string consists of the path from root context
     * delimited by "/" and name and context delimited by ".".
     *
     * For example, when <name0>.<context0>/.../<name[n]>.<context[n]>
     * is specified as the path, this operation returns all the
     * contexts and references under the "<name[n]>.<context[n]>"
     * context in a BindingList.
     *
     * @param string_name [in] The target path specified by a string
     * @param bl [out] Binding list under the specified path
     *
     * @endif
     */
    void list(const char* string_name,
              CosNaming::BindingList_var& bl);

    /*!
     * @if jp
     * @brief ^ꂽpXȉ̎w肳ꂽkind̃oCfBO擾
     *
     * Ŏw肳ꂽl[~O΃pXȉ̂Aw肳ꂽ kind
     * ׂẴoCfBO擾BpX́A[gReLX
     * gKw "/"AOƃReLXg "." ŋ؂ł
     * B
     *
     * Ⴆ΁A1ɃpX
     * <name0>.<context0>/.../<name[n]>.<context[n]> w肵A2
     *  kind "hoge" w肷ƁA<name[n]>.<context[n]> ̉ɃoC
     * hĂReLXg܂̓IuWFNgt@X̃Xg
     * ̂ kind  "hoge" ̂̂ BindingList ƂĕԂB
     *
     * Ⴆ΁A<name0>.<context0>/.../<name[n]>.<context[n]> w肷
     * <name[n]>.<context[n]> ̉ɃoChĂReLXg
     * ̓IuWFNgt@X̃Xg BindingList ƂĕԂ
     * B
     *
     * @param string_name [in] Ŏw肳^[Qbg̃pX
     * @param string_kind [in] Ŏw肳^[Qbgkind
     * @param bl [out] w肳ꂽpX̉̃oCfBOXg
     *
     * @else
     *
     * @brief Get all the binding with specified kind under given naming path
     *
     * This operation obtains all the binding list with specified kind
     * under specified absolute naming path. The path string consists
     * of the path from root context delimited by "/" and name and
     * context delimited by ".".
     *
     * For example, when <name0>.<context0>/.../<name[n]>.<context[n]>
     * is specified in the first argument as the path, and "hoge" is
     * specified the second argument as the kind, this operation
     * returns all the contexts and references under the
     * "<name[n]>.<context[n]>" context with kind "hoge" in a
     * BindingList.
     *
     * @param string_name [in] The target path specified by a string
     * @param string_kind [in] The target kind specified by a string
     * @param bl [out] Binding list under the specified path
     *
     * @endif
     */
    void listByKind(const char* string_name,
                    const char* string_kind,
                    CosNaming::BindingList_var& bl);
    
    //============================================================
    // interface of NamingContextExt
    //============================================================
    /*!
     * @if jp
     * @brief ^ꂽ NameComponent ̕\Ԃ
     *
     * w肳ꂽ NameComponent 𕶎ɕϊB
     *
     * @param name ϊΏ NameComponent
     *
     * @return ϊ
     *
     * @exception InvalidName  name ̖OsB
     *
     * @else
     * @brief Get string representation of given NameComponent
     *
     * Transform specified NameComponent into string representation.
     *
     * @param name The target NameComponent for transformation
     *
     * @return Trnasformation result of string representation
     *
     * @exception InvalidName The argument 'name' is invalid.
     *
     * @endif
     */
    char* toString(const CosNaming::Name& name)
      throw (SystemException, InvalidName);
    
    /*!
     * @if jp
     * @brief ^ꂽ\ NameComponent ɕ
     *
     * w肳ꂽ NameComponent ɕϊB
     *
     * @param string_name ϊΏە
     *
     * @return NameComponent ϊ
     *
     * @exception InvalidName  string_name sB
     *
     * @else
     * @brief Resolve given string representation to NameComponent
     *
     * Transform given string representation to NameComponent.
     *
     * @param string_name The target string representation to transform
     *
     * @return NameComponent The result of transformation
     *
     * @exception InvalidName The argument 'name' is invalid.
     *
     * @endif
     */
    CosNaming::Name toName(const char* string_name)
      throw (SystemException, InvalidName);
    
    /*!
     * @if jp
     * @brief ^ꂽ addre  string_name  URL\擾
     *
     * w肳ꂽAhXƖ̂URLɕϊB
     *
     * @param addr ϊΏۃAhX
     * @param string_name ϊΏۖ
     *
     * @return URL ϊ
     *
     * @exception InvalidAddress  addr sB
     * @exception InvalidName  string_name sB
     *
     * @else
     * @brief Get URL representation from given addr and string_name
     *
     * Convert specified addr and string_name into URL
     *
     * @param addr The target address for conversion
     * @param string_name The target name for conversion
     *
     * @return URL Conversion result
     *
     * @exception InvalidAddress The argument 'addr' is invalid.
     * @exception InvalidName The argument 'string_name' is invalid.
     *
     * @endif
     */
    char* toUrl(char* addr, char* string_name)
      throw (SystemException, InvalidAddress, InvalidName);
    
    /*!
     * @if jp
     * @brief ^ꂽ\ resolve IuWFNgԂ
     *
     * w肳ꂽ\resolveCIuWFNg擾B
     *
     * @param string_name 擾ΏۃIuWFNg\
     *
     * @return ꂽIuWFNg
     *
     * @exception NotFound r <c_1, c_2, ..., c_(n-1)> ݂ȂB
     * @exception CannotProceed 炩̗RŏpłȂB
     * @exception InvalidName  name ̖OsB
     * @exception AlreadyBound name <n>  Object łɃoChĂB
     *
     * @else
     * @brief Resolve from name of string representation and get object
     *
     * Resolve specified string representation and get object
     *
     * @param string_name The string representation of getting target object
     *
     * @return The resolved object
     *
     * @exception NotFound There is not <c_1, c_2, ..., c_(n-1)>.
     * @exception CannotProceed Processing cannot be continued for some reasons.
     * @exception InvalidName The argument 'name' is invalid.
     * @exception AlreadyBound The object of name<n> is already bound.
     *
     * @endif
     */
    CORBA::Object_ptr resolveStr(const char* string_name)
      throw (SystemException, NotFound, CannotProceed,
             InvalidName, AlreadyBound);
    
    //============================================================
    // Find functions
    //============================================================
    
    //    ObjectList find(const char* name, const char* kind);
    //    ObjectList findById(const char* name, const char* kind);
    //    ObjectList findByKind(const char* name, const char* kind);
    
    /*!
     * @if jp
     *
     * @brief IuWFNg̖OoCh܂͉
     *
     * w肳ꂽReLXgɑ΂ăIuWFNg NameComponent Ŏw肳ꂽ
     * ʒuɃoChB
     * ӏɊɑ̗vfoChς݂̏ꍇ́ÃoChςݗvf
     * 擾B
     *
     * @param context bind  resole ΏۃReLXg
     * @param name IuWFNgɕt閼O NameComponent
     * @param obj ֘At Object
     *
     * @return NameComponent Ŏw肳ꂽʒuɃoChĂIuWFNg
     *
     * @else
     * @brief Bind or resolve the given name component
     *
     * Bind object at the position that specified in NameComponent for the 
     * specified context.
     * When other elements are already bound at the same position, get the 
     * already bound element.
     *
     * @param context The context to bind or resole
     * @param name NameComponent applied to object
     * @param obj Object that is associated
     *
     * @return The object that is bound at position specified with NameComponent
     *
     * @endif
     */
    CORBA::Object_ptr bindOrResolve(CosNaming::NamingContext_ptr context,
				    const CosNaming::Name& name,
				    CORBA::Object_ptr obj);
    
    /*!
     * @if jp
     *
     * @brief OoCh܂͉
     *
     * w肳ꂽReLXgɑ΂ Context NameComponent Ŏw肳ꂽʒu
     * oChB
     * ӏɊɑ̗vfoChς݂̏ꍇ́ÃoChςݗvf
     * 擾B
     *
     * @param context bind  resole ΏۃReLXg
     * @param name ReLXgɕt閼O NameComponent
     * @param new_context ֘At Context
     *
     * @return NameComponent Ŏw肳ꂽʒuɃoChĂContext
     *
     * @else
     * @brief Bind or resolve the given name component
     *
     * Bind Context at the position that specified in NameComponent for the 
     * specified context.
     * When other elements are already bound at the same position, get the 
     * already bound element.
     *
     * @param context The context to bind or resole
     * @param name NameComponent applied to object
     * @param new_context Context that is associated
     *
     * @return The Context that is bound at the position specified with 
     *         NameComponent
     *
     * @endif
     */
    CosNaming::NamingContext_ptr
    bindOrResolveContext(CosNaming::NamingContext_ptr context,
			 const CosNaming::Name& name,
			 CosNaming::NamingContext_ptr new_context);
    
    /*!
     * @if jp
     * @brief OoCh܂͉
     *
     * w肳ꂽReLXgɑ΂ NameComponent Ŏw肳ꂽʒu
     * VKReLXgoChB
     * ӏɊɑ̗vfoChς݂̏ꍇ́ÃoChςݗvf
     * 擾B
     *
     * @param context bind  resole ΏۃReLXg
     * @param name VK쐬ReLXg̈ʒu\ NameComponent
     *
     * @return NameComponent Ŏw肳ꂽʒuɃoChĂContext
     *
     * @else
     * @brief Bind or resolve the given name component
     *
     * Bind new Context at the position that specified in NameComponent for the 
     * specified context.
     * When other elements are already bound at the same position, get the 
     * already bound element.
     *
     * @param context The context to bind or resole
     * @param name NameComponent that indicates the position of new context
     *
     * @return The Context that is bound at the position specified with 
     *         NameComponent
     *
     * @endif
     */
    CosNaming::NamingContext_ptr
    bindOrResolveContext(CosNaming::NamingContext_ptr context,
			 const CosNaming::Name& name);
    
    /*!
     * @if jp
     * @brief l[T[o̖O擾
     *
     * ݒ肵l[T[o̖O擾B
     *
     * @return l[T[o̖O
     *
     * @else
     * @brief Get the name of name server
     *
     * Get the configured name of name server
     *
     * @return The name of name server
     *
     * @endif
     */
    const char* getNameServer();
    
    /*!
     * @if jp
     * @brief [gReLXg擾
     *
     * ݒ肵l[T[õ[gReLXg擾B
     *
     * @return l[T[õ[gReLXg
     *
     * @else
     * @brief Get the root context
     *
     * Get the root context of the configured name server
     *
     * @return Root context ot name server
     *
     * @endif
     */
    CosNaming::NamingContext_ptr getRootContext();
    
    /*!
     * @if jp
     * @brief IuWFNgl[~OReLXgʂ
     *
     * w肵vfl[~OReLXgʂ
     *
     * @param obj ʑΏۗvf
     *
     * @return ʌ(l[~OReLXg:trueAȊO:false)
     *
     * @else
     * @brief Determine whether the object is NamingContext
     *
     * Determine whether the specified element is NamingContext
     *
     * @param obj The target element for determination
     *
     * @return Determination result (NamingContext:true, Else:false)
     *
     * @endif
     */
    bool isNamingContext(CORBA::Object_ptr obj);
    
    /*!
     * @if jp
     * @brief ^ꂽOl[~OReLXgǂ
     *
     * NameComponentŎw肵vfl[~OReLXgʂ
     *
     * @param name ʑΏNameComponent
     *
     * @return ʌ(l[~OReLXg:trueAȊO:false)
     *
     * @else
     * @brief Determine whether the given name component is NamingContext
     *
     * Determine whether the specified element is NameComponent
     *
     * @param name The target NameComponent for determination
     *
     * @return Determination result (NamingContext:true, Else:false)
     *
     * @endif
     */
    bool isNamingContext(const CosNaming::Name& name);
    
    /*!
     * @if jp
     * @brief ^ꂽOl[~OReLXgǂ
     *
     * Ŏw肵vfl[~OReLXgʂ
     *
     * @param string_name ʑΏە
     *
     * @return ʌ(l[~OReLXg:trueAȊO:false)
     *
     * @else
     * @brief Determine whether the given string name is NamingContext
     *
     * Determine whether the element specified by string name is NamingContext
     *
     * @param string_name The string representation for determination
     *
     * @return Determination result (NamingContext:true, Else:false)
     *
     * @endif
     */
    bool isNamingContext(const char* string_name);
    
    /*!
     * @if jp
     * @brief l[R|[lg̕Ԃ
     *
     * w肳ꂽ͈͂̃l[R|[lg擾B
     * Iʒuw肳ĂȂꍇ́AŌ̗vfl[R|[lg
     * ԂB
     *
     * @param name ΏNameComponent
     * @param begin 擾͈͊Jnʒu
     * @param end 擾͈͏Iʒu(ftHgl:-1)
     *
     * @return NameComponent 擾
     *
     * @else
     * @brief Get subset of given name component
     *
     * Get the name component in specified range.
     * Return the name component except the last element if the end 
     * position is not specified.
     *
     * @param name The target NameComponent for search
     * @param begin The beginning position for getting range
     * @param end The end position for getting range (The default value:-1)
     *
     * @return NameComponent Getting result
     *
     * @endif
     */
    CosNaming::Name subName(const CosNaming::Name& name,
			    CORBA::Long begin,
			    CORBA::Long end = -1);
    
  protected:
    /*!
     * @if jp
     * @brief l[R|[lg̕\擾
     *
     * w肵͈͂̃l[R|[lg̕\擾B
     * \́ANameComponent̍\{Nc[0], Nc[1], Nc[2]...}̏ꍇA
     *   Nc[0]id.Nc[0].kind/Nc[1]id.Nc[1].kind/Nc[2].id/Nc[2].kind...
     * Ƃ`Ŏ擾łB
     * 擾̒w肵ȏ̏ꍇ́A
     * w肵Ő؂̂ĂB
     *
     * @param name 擾ΏNameComponent
     * @param string_name 擾ʕ
     * @param slen 擾Ώەől
     *
     * @else
     * @brief Get string representation of name component
     *
     * Get string representation of the name component in specified range.
     * In string representation, if NameComponent consists of 
     * {Nc[0],Nc[1],Nc[2]...}, the format of 
     * Nc[0]id.Nc[0].kind/Nc[1]id.Nc[1].kind/Nc[2].id/Nc[2].kind...
     * will be got.
     * It is rounded by the specified length when the length of the got
     * string is over the specified length. 
     *
     * @param name The getting target NameComponent
     * @param string_name The string of getting result
     * @param slen The maximum length value of getting string
     *
     * @endif
     */
    void nameToString(const CosNaming::Name& name, char* string_name,
		      CORBA::ULong slen);
    /*!
     * @if jp
     * @brief l[R|[lg̕\̕擾
     *
     * w肵l[R|[lg𕶎ŕ\ꍇ̒擾B
     * \́ANameComponent̍\{Nc[0],Nc[1],Nc[2]...}̏ꍇA
     *   Nc[0]id.Nc[0].kind/Nc[1]id.Nc[1].kind/Nc[2].id/Nc[2].kind...
     * Ƃ`Ŏ擾łB
     *
     * @param name 擾ΏNameComponent
     *
     * @return w肵l[R|[lg̕񒷂
     *
     * @else
     * @brief Get string length of the name component's string representation
     *
     * Get string length of the name component's string representation.
     * In string representation, if NameComponent consists of 
     * {Nc[0],Nc[1],Nc[2]}, the format of 
     * Nc[0]id.Nc[0].kind/Nc[1]id.Nc[1].kind/Nc[2].id/Nc[2].kind
     * will be got.
     *
     * @param name The getting target NameComponent
     *
     * @return The string length value of specified component
     *
     * @endif
     */
    CORBA::ULong getNameLength(const CosNaming::Name& name);
    
    /*!
     * @if jp
     * @brief ̕
     *
     * w肵f~^ŕB
     *
     * @param input Ώە
     * @param delimiter pf~^
     * @param results 
     *
     * @return ̗vf
     *
     * @else
     * @brief Split of string
     *
     * Split string with specified delimiter.
     *
     * @param input The split target string
     * @param delimiter The delimiter for split
     * @param results Split result
     *
     * @return The number of split string elements
     *
     * @endif
     */
    unsigned int split(const std::string& input,
		       const std::string& delimiter,
		       std::vector<std::string>& results);
    
    /*!
     * @if jp
     * @brief ORB
     * @else
     * @brief ORB
     * @endif
     */
    CORBA::ORB_ptr m_ptrORB;
    
    /*!
     * @if jp
     * @brief l[T[o
     * @else
     * @brief Name of the name server
     * @endif
     */
    std::string m_nameServer;
    /*!
     * @if jp
     * @brief w肵l[T[õ[gReLXg
     * @else
     * @brief The root context of specified name server
     * @endif
     */
    CosNaming::NamingContextExt_var m_rootContext;
    
  private:
    CORBA::ULong m_blLength;
    
  }; // class CorbaNaming
}; // namespace RTC

#ifdef WIN32
#pragma warning( default : 4290 )
#endif

#endif // end of RTC_CORBANAMING_H
