// -*- C++ -*-
/*!
 * @file PortBase.h
 * @brief RTC's Port base class
 * @date $Date: 2008-01-14 07:56:44 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2010
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PortBase.h 3194 2018-01-23 10:20:54Z miyamoto $
 *
 */

#ifndef RTC_PORTBASE_H
#define RTC_PORTBASE_H

#include <rtm/RTC.h>

#include <string>
#include <vector>
#include <coil/Guard.h>
#include <coil/Mutex.h>
#include <rtm/idl/RTCSkel.h>
#include <rtm/CORBA_SeqUtil.h>
#include <rtm/NVUtil.h>
#include <rtm/SystemLogger.h>
#include <rtm/PortConnectListener.h>
#include <iostream>
#include <rtm/DirectPortBase.h>

#ifdef WIN32
#pragma warning( disable : 4290 )
#endif

namespace RTC
{
  class ConnectionCallback;

  /*!
   * @if jp
   * @class PortBase
   * @brief Port ̊NX
   *
   * RTC::Port ̊ƂȂNXB
   * RTC::Port ͂ق UML Port ̊TOpĂAقړ̂̂Ƃ݂Ȃ
   * ƂłBRT R|[lg̃RZvgɂẮA
   * Port ̓R|[lgɕtAR|[lg̃R|[lgƑݍp
   * sړ_łAʏ̃C^[tF[XƊ֘AtB
   * R|[lg Port ʂĊOɑ΂C^[tF[X񋟂܂͗v
   * 邱ƂłAPort͂̐ڑǗSB
   * <p>
   * Port ̋ۃNX́Aʏ RT R|[lgCX^Xɓ
   * A񋟁EvC^[tF[Xo^ART R|[lg
   * o^AOANZX\ Port Ƃċ@\邱Ƃz肵ĂB
   * <p>
   * RTC::Port  CORBA C^[tF[XƂĈȉ̃Iy[V񋟂B
   *
   * - get_port_profile()
   * - get_connector_profiles()
   * - get_connector_profile()
   * - connect()
   * - notify_connect()
   * - disconnect()
   * - notify_disconnect()
   * - disconnect_all()
   *
   * ̃NXł́ÃIy[V̎񋟂B
   * <p>
   * ̃Iy[V̂Aget_port_profile(), get_connector_profiles(),
   * get_connector_profile(), connect(), disconnect(), disconnect_all() ́A
   * TuNXɂēɐU镑ύXKvȂ߁AI[o[Ch
   * 邱Ƃ͐ȂB
   * <p>
   * notify_connect(), notify_disconnect() ɂẮATuNX񋟁Ev
   * C^[tF[X̎ނɉāAU镑ύXKv
   * \邪A𒼐ڃI[o[Ch邱Ƃ͐ꂸA
   * q notify_connect(), notify_disconnect() ̍ɂĂqׂʂ
   * ̊֐Ɋ֘A protected ֐I[o[Ch邱Ƃɂ
   * U镑ύX邱ƂB
   *
   * @since 0.4.0
   *
   * @else
   * @class PortBase
   * @brief Port base class
   *
   * This class is a base class of RTC::Port.
   * RTC::Port inherits a concept of RT-Component, and can be regarded as almost
   * the same as it. In the concept of RT-Component, Port is attached to the
   * component, can mediate interaction between other components and usually is
   * associated with some interfaces.
   * Component can provide or require interface for outside via Port, and the
   * Port plays a role to manage the connection.
   * <p>
   * Concrete class of Port assumes to be usually created at the same time that
   * RT-Component's instance is created, be registerd to RT-Component after
   * provided and required interfaces are registerd, and function as accessible
   * Port from outside.
   * <p>
   * RTC::Port provides the following operations as CORBA interface:
   *
   * - get_port_profile()
   * - get_connector_profiles()
   * - get_connector_profile()
   * - connect()
   * - notify_connect()
   * - disconnect()
   * - notify_disconnect()
   * - disconnect_all()
   *
   * This class provides implementations of these operations.
   * <p>
   * In these operations, as for get_port_profile(), get_connector_profiles(),
   * get_connector_profile(), connect(), disconnect() and disconnect_all(),
   * since their behaviors especially need not to be change in subclass, 
   * overriding is not recommended.
   * <p>
   * As for notify_connect() and notify_disconnect(), you may have to modify
   * behavior according to the kind of interfaces that subclass provides and
   * requires, however it is not recommended these are overriden directly.
   * In the section of notify_connect() and notify_disconnect() as described
   * below, it is recommended that you modify behavior by overriding the
   * protected function related to these functions.
   *
   * @since 0.4.0
   *
   * @endif
   */  
  class PortBase
    : public virtual POA_RTC::PortService,
      public virtual PortableServer::RefCountServantBase
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * PortBase ̃RXgN^ Port  name Ɏ菉s
     * ƓɁAg CORBA Object ƂĊAg PortProfile
     *  port_ref Ɏg̃IuWFNgt@Xi[B
     * Oɂ́A"." ȊO̕gp邱ƂłB
     *
     * @param name Port ̖O(ftHgl:"")
     *
     * @else
     *
     * @brief Constructor
     *
     * The constructor of the ProtBase class is given the name of this Port
     * and initialized. At the same time, the PortBase activates itself
     * as CORBA object and stores its object reference to the PortProfile's 
     * port_ref member.
     * Characters except "." can be used for the name of the port.
     *
     * @param name The name of Port (The default value:"")
     *
     * @endif
     */
    PortBase(const char* name = "");
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^ł́APortService CORBA IuWFNg deactivate 
     * sBdeactivateɍۂėO𓊂邱Ƃ͂ȂB
     *
     * @else
     *
     * @brief Destructor
     *
     * In the destructor, PortService CORBA object is deactivated.
     * This function never throws exception.
     *
     * @endif
     */
    virtual ~PortBase(void);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] PortProfile擾
     *
     * PortێPortProfileԂB̊֐ CORBA Iy[V
     * łACORBA ̃ǗKɏ]AĂяo͕Ԃ
     * PortProfile IuWFNĝӔCBPortProfile \
     * ͈ȉ̃o[B
     *
     * - name              [string ^] Port ̖OB
     * - interfaces        [PortInterfaceProfileList ^] Port ێ
     *                     PortInterfaceProfile ̃V[PX
     * - port_ref          [Port Object ^] Port g̃IuWFNgt@X
     * - connector_profile [ConnectorProfileList ^] Port ݕێ
     *                     ConnectorProfile ̃V[PX
     * - owner             [RTObject Object ^]  Port L
     *                     RTObject̃t@X
     * - properties        [NVList ^] ̑̃vpeBB
     *
     * @post ̊֐ĂяoƂɂԂύX邱Ƃ͂ȂB
     *
     * @return PortProfile
     *
     * @else
     *
     * @brief [CORBA interface] Get the PortProfile of the Port
     *
     * This operation returns the PortProfile of the Port. Since this
     * function is CORBA operation, callers have responsibility to
     * destruction of the returned PortProfile object according to the
     * CORBA memory management rules.
     *
     * PortProfile struct has the following members:
     *
     * - name              [string type] The name of the Port.
     * - interfaces        [PortInterfaceProfileList type] The sequence of 
     *                     PortInterfaceProfile owned by the Port
     * - port_ref          [Port Object type] The object reference of the Port.
     * - connector_profile [ConnectorProfileList type] The sequence of 
     *                     ConnectorProfile owned by the Port.
     * - owner             [RTObject Object type] The object reference of 
     *                     RTObject that is owner of the Port.
     * - properties        [NVList type] The other properties.
     *
     * @post This function never changes the state of the object.
     *
     * @return PortProfile of the Port
     *
     * @endif
     */
    virtual PortProfile* get_port_profile()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief PortProfile 擾B
     *
     * ̊֐́AIuWFNgɕێĂ PortProfile 
     * const QƂԂ const ֐łB
     * 
     * @post ̊֐ĂяoƂɂԂύX邱Ƃ͂ȂB
     *
     * @return PortProfile
     *
     * @else
     *
     * @brief Get the PortProfile of the Port
     *
     * This function is a const function that returns a const
     * reference of the PortProfile stored in this Port.
     *
     * @post This function never changes the state of the object.
     *
     * @return PortProfile
     *
     * @endif
     */
    const PortProfile& getPortProfile() const;
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] ConnectorProfileList擾
     *
     * Portێ ConnectorProfile  sequence ԂB̊֐
     * CORBA Iy[VłACORBA ̃ǗKɏ]AĂяo
     * ͕Ԃ ConnectorProfileList IuWFNĝӔC
     * B
     *
     * ConnectorProfile  Port Ԃ̐ڑvt@Cێ\̂łA
     * ڑPortԂŏsA֘A邷ׂĂ Port œ̒l
     * ێB
     * ConnectorProfile ͈ȉ̃o[ێĂB
     *
     * - name         [string ^] ̃RlN^̖OB
     * - connector_id [string ^] j[NID
     * - ports        [Port sequnce] ̃RlN^Ɋ֘A Port ̃IuWFNg
     *                t@X̃V[PXB
     * - properties   [NVList ^] ̑̃vpeBB
     *
     * @post ̊֐ĂяoƂɂԂύX邱Ƃ͂ȂB
     *
     * @return  Port ێ ConnectorProfile
     *
     * @else
     *
     * @brief [CORBA interface] Get the ConnectorProfileList of the Port
     *
     * This operation returns a list of the ConnectorProfiles of the
     * Port.  Since this function is CORBA operation, callers have
     * responsibility to destruction of the returned ConnectorProfileList
     * object according to the CORBA memory management rules.
     *
     * ConnectorProfile includes the connection information that
     * describes relation between (among) Ports, and Ports exchange
     * the ConnectionProfile on connection process and hold the same
     * information in every Port.  ConnectionProfile has the following
     * members:
     *
     * - name         [string type] The name of the connection.
     * - connector_id [string type] Unique identifier.
     * - ports        [Port sequnce] The sequence of Port's object reference
     *                that are related the connection.
     * - properties   [NVList type] The other properties.
     *
     * @post This function never changes the state of the object.
     *
     * @return ConnectorProfileList of the Port
     *
     * @endif
     */
    virtual ConnectorProfileList* get_connector_profiles()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] ConnectorProfile 擾
     *
     * connector_id Ŏw肳ꂽ ConnectorProfile ԂB̊֐
     * CORBA Iy[VłACORBA ̃ǗKɏ]AĂяo
     * ͕Ԃ ConnectorProfile IuWFNĝӔCB
     *
     * @pre ɗ^ connector_id ͗LȕłȂ΂ȂȂB
     * 󕶎w肵ꍇA܂͎w肵 connector_id 
     * ConnectorProfile Ȃꍇ́A ConnectorProfile 
     * ԂB
     *
     * @post ̊֐ĂяoƂɂԂύX邱Ƃ͂ȂB
     *
     * @param connector_id ConnectorProfile  ID
     * @return connector_id Ŏw肳ꂽ ConnectorProfile
     *
     * @else
     *
     * @brief [CORBA interface] Get the ConnectorProfile
     *
     * This operation returns the ConnectorProfiles specified
     * connector_id.  Since this function is CORBA operation, callers
     * have responsibility to destruction of the returned
     * ConnectorProfile object according to the CORBA memory
     * management rules.
     *
     * If ConnectorProfile with specified connector_id is not included,
     * empty ConnectorProfile is returned.
     *
     * @post This function never changes the state of the object.
     *
     * @param connector_id ID of the ConnectorProfile
     * @return the ConnectorProfile identified by the connector_id
     *
     * @endif
     */
    virtual ConnectorProfile* get_connector_profile(const char* connector_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̐ڑs
     *
     * ^ꂽ ConnectoionProfile ̏ɊÂAPortԂ̐ڑm
     * B̊֐͎ɃAvP[VvOc[Ăяo
     * ƂOƂĂB
     * 
     * @pre AvP[VvÓAR|[lgԂ̕
     * Port ڑ邽߂ɁAK؂ȒlZbg ConnectorProfile 
     * connect() ̈Ƃė^ČĂяoȂ΂ȂȂB
     *
     * @pre connect() ɗ^ ConnectorProfile ̃o[̂A
     * name, ports, properties o[ɑ΂ăf[^ZbgȂ
     * ȂȂBconnector_id ɂ͒ʏ󕶎ݒ肷邩AKUUID
     * Őݒ肷KvB
     *
     * @pre ConnectorProfile::name ͐ڑɂ閼O CORBA::string
     * ^Ɋi[łCӂ̕łKvB
     * 
     * @pre ConnectorProfile::connector_id ׂ͂Ă̐ڑɑ΂Ĉӂ
     * ID (ʏUUID) i[BUUID̐ݒ connect() ֐ōs
     * ̂ŁAĂяo͋󕶎ݒ肷B̐ڑƓUUID
     * ݒ肵 connect() Ăяoꍇɂ PRECONDITION_NOT_MET G[
     * ԂBÅgŊ̐ڑvt@CXV邽
     * Ɋ UUID ݒ肵ČĂяogp@p\B
     *
     * @pre ConnectorProfile::ports  RTC::PortService ̃V[PXŁA
     * ڑ\ʏ2ȏ̃|[g̃IuWFNgQƂK
     * vBOƂāA|[g̃IuWFNgQƂ1i[
     * connect()ĂяoƂŁA|[g̃C^[tF[X擾
     * Aȃ|[g(CORBARTC::PortServiceȊȎ)ɑ΂Đ
     * sꍇB
     *
     * @pre ConnectorProfile::properties ̓|[gɊ֘AtꂽC^[
     * tF[Xɑ΂vpeB^邽߂ɎgpBvpeB́A
     * string ^L[AAny ^lƂĂyÃV[PXłAl
     * ɂ͔CӂCORBAf[^^i[ł邪A\Ȍ string ^Ƃ
     * Ċi[邱ƂB
     *
     * @pre ȏ connect() Ăяoɐݒ肷 ConnectorProfile ̃
     * o܂Ƃ߂ƈȉ̂悤ɂȂB
     *
     * - ConnectorProfile::name: Cӂ̐ڑ
     * - ConnectorProfile::connector_id: 󕶎
     * - ConnectorProfile::ports: 1ȏ̃|[g
     * - ConnectorProfile::properties: C^[tF[Xɑ΂vpeB
     *
     * @post connect() ֐́AConnectorProfile::portsɊi[ꂽ|[
     * gV[PX̐擪̃|[gɑ΂ notify_connect() ĂԁB
     *
     * @post notify_connect()  ConnectorProfile::ports Ɋi[ꂽ|[
     * g notify_connect() JXP[hĂяoB̃JXP[h
     * ĂяóArnotify_connect() ŃG[oĂ|[g̃Iu
     * WFNgQƂLłAKׂẴ|[gɑ΂čs
     * Ƃۏ؂BLłȂIuWFNgQƂV[PXɑ
     * ꍇÃ|[gXLbvāÃ|[gɑ΂
     * notify_connect() ĂяoB
     *
     * @post connect() ֐́Anotify_connect()̖߂lRTC_OKł΁A
     * RTC_OK ԂB̎_Őڑ͊BRTC_OKȊO
     * ̏ꍇ́A̐ڑIDɑ΂disconnect()ĂяoڑA
     * notify_connect() ԂG[^[R[ĥ܂ܕԂB
     * 
     * @post connect() ̈Ƃēn ConnectorProfile ɂ́A
     * ConnectorProfile::connector_id сAr̃|[g
     * publishInterfaces() ŌJ|[gC^[tF[X̊e
     * i[ĂBconnect() ѓr notify_connect() 
     * ConnectorProfile::{name, ports} ύX邱Ƃ͂ȂB
     *  
     * @param connector_profile ConnectorProfile
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Connect the Port
     *
     * This operation establishes connection according to the given
     * ConnectionProfile inforamtion. This function is premised on
     * calling from mainly application program or tools.
     *
     * @pre To establish the connection among Ports of RT-Components,
     * application programs must call this operation giving
     * ConnectorProfile with valid values as an argument.
     *
     * @pre Out of ConnectorProfile member variables, "name", "ports"
     * and "properties" members shall be set valid
     * data. "connector_id" shall be set as empty string value or
     * valid string UUID value.
     *
     * @pre ConnectorProfile::name that is connection identifier shall
     * be any valid CORBA::string.
     * 
     *
     * @pre ConnectorProfile::connector_id shall be set unique
     * identifier (usually UUID is used) for all connections. Since
     * UUID string value is usually set in the connect() function,
     * caller should just set empty string. If the connect() is called
     * with the same UUID as existing connection, this function
     * returns PRECONDITION_NOT_MET error. However, in order to update
     * the existing connection profile, the "connect()" operation with
     * existing connector ID might be used as valid method by future
     * extension
     *
     * @pre ConnectorProfile::ports, which is sequence of
     * RTC::PortService references, shall store usually two or more
     * ports' references. As exceptions, the "connect()" operation
     * might be called with only one reference in ConnectorProfile, in
     * case of just getting interfaces information from the port, or
     * connecting a special port (i.e. the peer port except
     * RTC::PortService on CORBA).
     *
     * @pre ConnectorProfile::properties might be used to give certain
     * properties to the service interfaces associated with the port.
     * The properties is a sequence variable with a pair of key string
     * and Any type value. Although the A variable can store any type
     * of values, it is not recommended except string.
     *
     * @pre The following is the summary of the ConnectorProfile
     * member to be set when this operation is called.
     *
     * - ConnectorProfile::name: The any name of connection
     * - ConnectorProfile::connector_id: Empty string
     * - ConnectorProfile::ports: One or more port references
     * - ConnectorProfile::properties: Properties for the interfaces
     *
     * @post connect() operation will call the first port in the
     * sequence of the ConnectorProfile.
     *
     * @post "noify_connect()"s perform cascaded call to the ports
     * stored in the ConnectorProfile::ports by order. Even if errors
     * are raised by intermediate notify_connect() operation, as long
     * as ports' object references are valid, it is guaranteed that
     * this cascaded call is completed in all the ports.  If invalid
     * or dead ports exist in the port's sequence, the ports are
     * skipped and notify_connect() is called for the next valid port.
     *
     * @post connect() function returns RTC_OK if all the
     * notify_connect() return RTC_OK. At this time the connection is
     * completed.  If notify_connect()s return except RTC_OK,
     * connect() calls disconnect() operation with the connector_id to
     * destruct the connection, and then it returns error code from
     * notify_connect().
     *
     * @post The ConnectorProfile argument of the connect() operation
     * returns ConnectorProfile::connector_id and various information
     * about service interfaces that is published by
     * publishInterfaces() in the halfway ports. The connect() and
     * halfway notify_connect() functions never change
     * ConnectorProfile::{name, ports}.
     *
     * @param connector_profile The ConnectorProfile.
     * @return ReturnCode_t The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t connect(ConnectorProfile& connector_profile)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̐ڑʒms
     *
     * ̃Iy[V́APortԂ̐ڑsۂɁAPortԂœI
     * ɌĂ΂Iy[VłāAʏAvP[VvO
     * APortȊORTC֘AIuWFNgĂяo邱Ƃ͑z肳
     * ĂȂB
     *
     * notify_connect() ̂̓ev[g\bhp^[ƂāATu
     * NXŎ邱ƂO publishInterfaces(),
     * subscribeInterfaces() 2̊֐ŌĂяoB̎菇
     * ȉ̒ʂłB
     *
     * - publishInterfaces(): C^[tF[X̌J
     * - connectNext():  Port  notify_connect() ̌Ăяo
     * - subscribeInterfaces(): C^[tF[X̎擾
     * - ڑ̕ۑ
     *
     * notify_connect()  ConnectorProfile::ports Ɋi[Ă
     * Port ̏ɏ]āAJXP[hɌĂяosƂɂAC
     * ^[tF[X̌JƎ擾֘AׂẴ|[gɑ΂čsB
     * ̃JXP[hĂяo͓rŒf邱Ƃ͂ȂAK
     * ConnectorProfile::ports Ɋi[ĂS|[gɑ΂čsB
     *
     * @pre notify_connect()  ConnectorProfile::ports Ɋi[
     *  Port QƃXĝAY Port g̎QƂ̎Ɋi[
     *  Port ɑ΂ notify_connect() ĂяoB
     * ConnectorProfile::ports ɂ͓Y Port ̎QƂi[ĂKv
     * BAg̎QƂi[ĂȂꍇȂ̏ɂ
     * G[㏑Ȃ΁ABAD_PARAMETER G[ԂB
     *
     * @pre Ăяo ConnectorProfile::connector_id ɂ͈ӂID
     *  UUID ێĂKvBʏ connector_id 
     * connect() ֐ɂ^A󕶎̏ꍇ͓͖`łB
     *
     * @post ConnectorProfile::name, ConnectorProfile::connector_id,
     * ConnectorProfile::ports  notify_connect() ̌Ăяoɂ
     * 邱Ƃ͂ȂsςłB
     *
     * @post ConnectorProfile::properties  notify_connect() ̓ŁA
     * Y Port T[rXC^[tF[XɊւ𑼂 Port
     * ɓ`邽߂ɁAvpeB񂪏܂B
     *
     * @post ȂAConnectorProfile::ports ̃Xg̍ŏ Port 
     * notify_connet() I_ł́AׂĂ̊֘A Port 
     * notify_connect() ̌ĂяoBpublishInterfaces(),
     * connectNext(), subscribeInterfaces() ѐڑ̕ۑ̂
     * ꂩ̒iKŃG[ꍇłAG[R[h͓Iɕێ
     * ĂAŏɐG[̃G[R[hԂB
     *
     * @param connector_profile ConnectorProfile
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Notify the Ports connection
     *
     * This operation is usually called from other ports' connect() or
     * notify_connect() operations when connection between ports is
     * established.  This function is not premised on calling from
     * other functions or application programs.
     *
     * According to the template method pattern, the notify_connect()
     * calls "publishInterfaces()" and "subsctiveInterfaces()"
     * functions, which are premised on implementing in the
     * subclasses. The processing sequence is as follows.
     *
     * - publishInterfaces(): Publishing interface information
     * - connectNext(): Calling notify_connect() of the next port
     * - subscribeInterfaces(): Subscribing interface information
     * - Storing connection profile
     *
     * According to the order of port's references stored in the
     * ConnectorProfile::ports, publishing interface information to
     * all the ports and subscription interface information from all
     * the ports is performed by "notify_connect()"s.  This cascaded
     * call never aborts in the halfway operations, and calling
     * sequence shall be completed for all the ports.
     *
     * @pre notify_connect() calls notify_connect() for the port's
     * reference that is stored in next of this port's reference in
     * the sequence of the ConnectorProfile::ports. Therefore the
     * reference of this port shall be stored in the
     * ConnectorProfile::ports. If this port's reference is not stored
     * in the sequence, BAD_PARAMETER error will be returned, except
     * the return code is overwritten by other operations.
     *
     * @pre UUID shall be set to ConnectorProfile::connector_id as a
     * unique identifier when this operation is called.  Usually,
     * connector_id is given by a connect() function and, the behavior
     * is undefined in the case of a null character.
     *
     * @post ConnectorProfile::name, ConnectorProfile::connector_id,
     * ConnectorProfile::ports are invariant, and they are never
     * rewritten by notify_connect() operations.
     *
     * @post In order to transfer interface information to other
     * ports, interface property information is stored into the
     * ConnectorProfile::properties.
     *
     * @post At the end of notify_connect() operation for the first
     * port stored in the ConnectorProfile::ports sequence, the
     * related ports' notify_connect() invocations complete. Even if
     * errors are raised at the halfway of publishInterfaces(),
     * connectNext(), subscribeInterfaces() and storing process of
     * ConnectorProfile, error codes are saved and the first error is
     * returned.
     *
     * @param connector_profile The ConnectorProfile.
     * @return ReturnCode_t The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t notify_connect(ConnectorProfile& connector_profile)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̐ڑ
     *
     * ̃Iy[V͗^ꂽ connector_id ɑΉڑ
     * Bconnector_id ͒ʏAVXeŜɂĈӂ UUID ̕
     * łAO connect()/notify_connect() ̌Ăяoɂm
     * ꂽڑvt@C ConnectorProfile::connector_id ɑΉ
     * B
     *
     * @pre connector_id  Port ێ ConnectorProfile ̏Ȃ
     *  ID Ɉv镶łȂ΂ȂȂBY Port 
     * ConnectorProfile ̃Xg connector_id Ɠ ID 
     * ConnectorProfile ݂Ȃ΂̊֐ BAD_PARAMETER G[
     * ԂB
     *
     * @pre connector_id Ɠ ID  ConnectorProfile::ports ɂ
     * L Port ̎QƂ܂܂ĂȂ΂ȂȂB
     *
     * @post disconnect() ֐́AConnectorProfile::ports  Port ̎Q
     * ƃXg̐擪ɑ΂āAnotify_disconnect() ĂяoBQƂ
     * łȂǁAnotify_disconnect() ̌ĂяoɎsꍇɂ́A
     * QƃXg̐擪珇Ԃɐ܂ notify_disconnect() ̌
     * яoBnotify_disconnect() ̌ĂяoɈł΁A
     * notify_disconnect() ̕ԋpl̂܂ܕԂAȂ
     * ꍇɂ RTC_ERROR G[ԂB
     * 
     * @param connector_id ConnectorProfile  ID
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Disconnect the Port
     *
     * This operation destroys connection between this port and the
     * peer port according to given connector_id. Usually connector_id
     * should be a UUID string that is unique in the system.  And the
     * connection, which is established by connect()/notify_connect()
     * functions, is identified by the ConnectorProfile::connector_id.
     *
     * @pre connector_id shall be a character string which is same
     * with ID of at least one of the ConnectorProfiles stored in this
     * port. If ConnectorProfile that has same ID with the given
     * connector_id does not exist in the list of ConnectorProfile,
     * this operation returns BAD_PARAMTER error.
     *
     * @pre ConnectorProfile::ports that is same ID with given
     * connector_id shall store the valid ports' references.
     *
     * @post disconnect() function invokes the notify_disconnect() for
     * the port that is stored in the first of the
     * ConnectorProfile::ports. If notify_disconnect() call fails for
     * the first port, It tries on calling "notify_disconnect()" in
     * order for ports stored in ConnectorProfile::ports until the
     * operation call is succeeded. If notify_disconnect() succeeded
     * for at least one port, it returns return code from
     * notify_disconnect(). If none of notify_connect() call
     * succeeded, it returns RTC_ERROR error.
     *
     * @param connector_id The ID of the ConnectorProfile.
     * @return ReturnCode_t The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t disconnect(const char* connector_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̐ڑʒms
     *
     * ̃Iy[V́APortԂ̐ڑsۂɁAPortԂœ
     * IɌĂ΂Iy[VłAʏAvP[VvO
     * A Port ȊO RTC ֘AIuWFNgĂяo邱Ƃ
     * z肳ĂȂB
     *
     * notify_disconnect() ̂̓ev[g\bhp^[ƂāAT
     * uNXŎ邱ƂO unsubscribeInterfaces() ֐
     * ŌĂяoB̎菇͈ȉ̒ʂłB
     *
     * - ConnectorProfile ̌
     * -  Port  notify_disconnect() Ăяo
     * - unsubscribeInterfaces()
     * - ConnectorProfile ̍폜
     *
     * notify_disconnect()  ConnectorProfile::ports Ɋi[Ă
     * Port ̏ɏ]āAJXP[hɌĂяosƂɂA
     * ׂ̉Ă Port ɒʒmB
     *
     * @pre Port ͗^ꂽ connector_id ɑΉ ConnectorProfile
     * ێĂȂ΂ȂȂB
     *
     * @post connector_id ɑΉ ConnectorProfile Ȃ
     * BAD_PARAMETER G[ԂB
     *
     * @post JXP[hĂяosۂɂ ConnectorProfile::ports 
     * ێĂ Port ̎QƃXĝAg̎QƂ̎̎QƂɑ
     *  notify_disconnect() ĂяoǍĂяoŗO
     * ꍇɂ́AĂяoXLbvXg̎̎QƂɑ΂
     * notify_disconnect() ĂяoBĂяoɐȂꍇA
     * RTC_ERROR G[R[hԂB
     *
     * @post ȂAConnectorProfile::ports ̃Xg̍ŏ Port 
     * notify_disconnet() I_ł́AׂĂ̊֘A Port 
     * notify_disconnect() ̌ĂяoB
     * 
     * @param connector_id ConnectorProfile  ID
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Notify the Ports disconnection
     *
     * This operation is invoked between Ports internally when the
     * connection is destroied. Generally it is not premised on
     * calling from application programs or RTC objects except Port
     * object.
     *
     * According to the template method pattern, the
     * notify_disconnect() calls unsubsctiveInterfaces() function,
     * which are premised on implementing in the subclasses. The
     * processing sequence is as follows.
     *
     * - Searching ConnectorProfile
     * - Calling notify_disconnect() for the next port
     * - Unsubscribing interfaces
     * - Deleting ConnectorProfile
     *
     * notify_disconnect() notifies disconnection to all the ports by
     * cascaded call to the stored ports in the
     * ConnectorProfile::ports in order.
     *
     * @pre The port shall store the ConnectorProfile having same id
     * with connector_id.
     *
     * @post If ConnectorProfile of same ID with connector_id does not
     * exist, it returns BAD_PARAMETER error.
     *
     * @post For the cascaded call, this operation calls
     * noify_disconnect() for the port that is stored in the next of
     * this port in the ConnectorProfile::ports.  If the operation
     * call raises exception for some failure, it tries to call
     * notify_disconnect() and skips until the operation succeeded.
     * If none of operation call succeeded, it returns RTC_ERROR.
     *
     * @post At the end of notify_disconnect() operation for the first
     * port stored in the ConnectorProfile::ports sequence, the
     * related ports' notify_disconnect() invocations complete.
     *
     * @param connector_id The ID of the ConnectorProfile.
     * @return ReturnCode_t The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t notify_disconnect(const char* connector_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̑Sڑ
     *
     * ̃Iy[V͂ Port Ɋ֘ASĂ̐ڑB
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Disconnect the All Ports
     *
     * This operation destroys all connections associated with this Port.
     *
     * @return ReturnCode_t The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t disconnect_all()
      throw (CORBA::SystemException);
    
    //============================================================
    // Local operations
    //============================================================
    /*!
     * @if jp
     *
     * @brief Port ̑SẴC^[tF[X activates 
     *
     * Port ɓo^ĂSẴC^[tF[X activate B
     *
     * @else
     *
     * @brief Activate all Port interfaces
     *
     * This operation activate all interfaces that is registered in the
     * ports.
     *
     * @endif
     */
    virtual void activateInterfaces() = 0;

    /*!
     * @if jp
     *
     * @brief SĂ Port ̃C^[tF[X deactivates 
     *
     * Port ɓo^ĂSẴC^[tF[X deactivate B
     *
     * @else
     *
     * @brief Deactivate all Port interfaces
     *
     * This operation deactivate all interfaces that is registered in the
     * ports.
     *
     * @endif
     */
    virtual void deactivateInterfaces() = 0;

    /*!
     * @if jp
     * @brief Port ̖Oݒ肷
     *
     * Port ̖Oݒ肷B̖O Port ێ PortProfile.name
     * ɔfB
     *
     * @param name Port ̖O
     *
     * @else
     * @brief Set the name of this Port
     *
     * This operation sets the name of this Port. The given Port's name is
     * applied to Port's PortProfile.name.
     *
     * @param name The name of this Port.
     *
     * @endif
     */
    void setName(const char* name);

     /*!
     * @if jp
     * @brief Port ̖O擾
     *
     * Port ̖O擾B
     *
     * @return Port ̖O
     *
     * @else
     * @brief Get the name of this Port
     *
     * This operation returns the name of this Port.
     *
     * @return The name of this Port.
     *
     * @endif
     */
   const char* getName() const;

    /*!
     * @if jp
     * @brief PortProfile擾
     *
     * Portێ PortProfile  const QƂԂB
     *
     * @return  Port  PortProfile
     *
     * @else
     * @brief Get the PortProfile of the Port
     *
     * This operation returns const reference of the PortProfile.
     *
     * @return PortProfile of the Port
     *
     * @endif
     */
    const PortProfile& getProfile() const;
    
    /*!
     * @if jp
     *
     * @brief Port ̃IuWFNgQƂݒ肷
     *
     * ̃Iy[V Port  PortProfile ɂ Port g
     * IuWFNgQƂݒ肷B
     *
     * @param port_ref  Port ̃IuWFNgQ
     *
     * @else
     *
     * @brief Set the object reference of this Port
     *
     * This operation sets the object reference itself
     * to the Port's PortProfile.
     *
     * @param port_ref The object reference of this Port.
     *
     * @endif
     */
    void setPortRef(PortService_ptr port_ref);
    
    /*!
     * @if jp
     *
     * @brief Port ̃IuWFNgQƂ擾
     *
     * ̃Iy[V Port  PortProfile ێĂ
     *  Port g̃IuWFNgQƂ擾B
     *
     * @return  Port ̃IuWFNgQ
     *
     * @else
     *
     * @brief Get the object reference of this Port
     *
     * This operation returns the object reference
     * that is stored in the Port's PortProfile.
     *
     * @return The object reference of this Port.
     *
     * @endif
     */
    PortService_ptr getPortRef() const;
    
    /*!
     * @if jp
     *
     * @brief Port  owner  RTObject w肷
     *
     * ̃Iy[V Port  PortProfile.owner ݒ肷B
     *
     * @param owner  Port L RTObject ̎Q
     *
     * @else
     *
     * @brief Set the owner RTObject of the Port
     *
     * This operation sets the owner RTObject of this Port.
     *
     * @param owner The owner RTObject's reference of this Port
     *
     * @endif
     */
    void setOwner(RTObject_ptr owner);

    //============================================================
    // callbacks
    //============================================================
    /*!
     * @if jp
     *
     * @brief C^[tF[XJۂɌĂ΂R[obNZbg
     *
     * ̃Iy[V́Ã|[gڑɁA|[ggT[
     * rXC^[tF[XJ^C~OŌĂ΂R[ob
     * Nt@N^ZbgB
     *
     * R[obNt@N^̏ĹAĂяoɂAIuWFNg
     * KvȂȂɉ̂̂͌Ăяo̐ӔCłB
     *
     * ̃R[obNt@N^́APortBaseNX̉z֐ł
     * publishInterfaces() Ă΂ꂽƂɁA ConnectorProfile 
     * ƂɌĂяoB̃R[obN𗘗pāA
     * publishInterfaces() J ConnectorProfile ύX邱Ƃ
     * \ł邪Aڑ֌W̕sȂ悤AConnectorProfile 
     * ύXɂ͒ӂvB
     *
     * @param on_publish ConnectionCallback ̃TuNXIuWFNg̃|C^
     *
     * @else
     *
     * @brief Setting callback called on publish interfaces
     *
     * This operation sets a functor that is called after publishing
     * interfaces process when connecting between ports.
     *
     * Since the ownership of the callback functor object is owned by
     * the caller, it has the responsibility of object destruction.
     * 
     * The callback functor is called after calling
     * publishInterfaces() that is virtual member function of the
     * PortBase class with an argument of ConnectorProfile type that
     * is same as the argument of publishInterfaces() function.
     * Although by using this functor, you can modify the ConnectorProfile
     * published by publishInterfaces() function, the modification
     * should be done carefully for fear of causing connection
     * inconsistency.
     *
     * @param on_publish a pointer to ConnectionCallback's subclasses
     *
     * @endif
     */
    void setOnPublishInterfaces(ConnectionCallback* on_publish);

    /*!
     * @if jp
     *
     * @brief C^[tF[X擾ۂɌĂ΂R[obNZbg
     *
     * ̃Iy[V́Ã|[gڑɁÃ|[gT[
     * rXC^[tF[X擾^C~OŌĂ΂R[ob
     * Nt@N^ZbgB
     *
     * R[obNt@N^̏ĹAĂяoɂAIuWFNg
     * KvȂȂɉ̂̂͌Ăяo̐ӔCłB
     *
     * ̃R[obNt@N^́APortBaseNX̉z֐ł
     * subscribeInterfaces() Ă΂OɁA ConnectorProfile 
     * ƂɌĂяoB̃R[obN𗘗pāA
     * subscribeInterfaces() ɗ^ ConnectorProfile ύX邱Ƃ
     * \ł邪Aڑ֌W̕sȂ悤AConnectorProfile 
     * ύXɂ͒ӂvB
     *
     * @param on_subscribe ConnectionCallback ̃TuNXIuWFNg̃|C^
     *
     * @else
     *
     * @brief Setting callback called on publish interfaces
     *
     * This operation sets a functor that is called before subscribing
     * interfaces process when connecting between ports.
     *
     * Since the ownership of the callback functor object is owned by
     * the caller, it has the responsibility of object destruction.
     * 
     * The callback functor is called before calling
     * subscribeInterfaces() that is virtual member function of the
     * PortBase class with an argument of ConnectorProfile type that
     * is same as the argument of subscribeInterfaces() function.
     * Although by using this functor, you can modify ConnectorProfile
     * argument for subscribeInterfaces() function, the modification
     * should be done carefully for fear of causing connection
     * inconsistency.
     *
     * @param on_subscribe a pointer to ConnectionCallback's subclasses
     *
     * @endif
     */
    void setOnSubscribeInterfaces(ConnectionCallback* on_subscribe);

    /*!
     * @if jp
     *
     * @brief ڑɌĂ΂R[obNZbg
     *
     * ̃Iy[V́Ã|[gڑɌĂ΂AR[ob
     * Nt@N^ZbgB
     *
     * R[obNt@N^̏ĹAĂяoɂAIuWFNg
     * KvȂȂɉ̂̂͌Ăяo̐ӔCłB
     *
     * ̃R[obNt@N^́A|[g̐ڑs֐ł
     * notify_connect() ̏IOɁAڑIۂɌ
     * ĂяoR[obNłBڑ̉ߒŃG[
     * ꍇɂ͌ĂяoȂB
     * 
     * ̃R[obNt@N^ notify_connect()  out p[^
     * ƂĕԂ̂Ɠ ConnectorProfile ƂƂɌĂяôŁA
     * ̐ڑɂČJꂽׂẴC^[tF[X𓾂邱
     * łB̃R[obN𗘗pāAnotify_connect() Ԃ
     * ConnectorProfile ύX邱Ƃ\ł邪Aڑ֌W̕s
     * Ȃ悤AConnectorProfile ̕ύXɂ͒ӂvB
     *
     * @param on_subscribe ConnectionCallback ̃TuNXIuWFNg̃|C^
     *
     * @else
     *
     * @brief Setting callback called on connection established
     *
     * This operation sets a functor that is called when connection
     * between ports established.
     *
     * Since the ownership of the callback functor object is owned by
     * the caller, it has the responsibility of object destruction.
     * 
     * The callback functor is called only when notify_connect()
     * function successfully returns. In case of error, the functor
     * will not be called.
     *
     * Since this functor is called with ConnectorProfile argument
     * that is same as out-parameter of notify_connect() function, you
     * can get all the information of published interfaces of related
     * ports in the connection.  Although by using this functor, you
     * can modify ConnectorProfile argument for out-paramter of
     * notify_connect(), the modification should be done carefully for
     * fear of causing connection inconsistency.
     *
     * @param on_subscribe a pointer to ConnectionCallback's subclasses
     *
     * @endif
     */
    void setOnConnected(ConnectionCallback* on_connected);

    /*!
     * @if jp
     *
     * @brief C^[tF[XۂɌĂ΂R[obNZbg
     *
     * ̃Iy[V́Ã|[gڑɁÃ|[gT[
     * rXC^[tF[X^C~OŌĂ΂R[ob
     * Nt@N^ZbgB
     *
     * R[obNt@N^̏ĹAĂяoɂAIuWFNg
     * KvȂȂɉ̂̂͌Ăяo̐ӔCłB
     *
     * ̃R[obNt@N^́APortBaseNX̉z֐ł
     * unsubscribeInterfaces() Ă΂OɁA ConnectorProfile 
     * ƂɌĂяoB̃R[obN𗘗pāA
     * unsubscribeInterfaces() ɗ^ ConnectorProfile ύX邱Ƃ
     * \ł邪Aڑ֌W̕sȂ悤AConnectorProfile 
     * ύXɂ͒ӂvB
     *
     * @param on_unsubscribe ConnectionCallback ̃TuNXIuWFN
     * g̃|C^
     *
     * @else
     *
     * @brief Setting callback called on unsubscribe interfaces
     *
     * This operation sets a functor that is called before unsubscribing
     * interfaces process when disconnecting between ports.
     *
     * Since the ownership of the callback functor object is owned by
     * the caller, it has the responsibility of object destruction.
     * 
     * The callback functor is called before calling
     * unsubscribeInterfaces() that is virtual member function of the
     * PortBase class with an argument of ConnectorProfile type that
     * is same as the argument of unsubscribeInterfaces() function.
     * Although by using this functor, you can modify ConnectorProfile
     * argument for unsubscribeInterfaces() function, the modification
     * should be done carefully for fear of causing connection
     * inconsistency.
     *
     * @param on_unsubscribe a pointer to ConnectionCallback's subclasses
     *
     * @endif
     */
    void setOnUnsubscribeInterfaces(ConnectionCallback* on_subscribe);

    /*!
     * @if jp
     *
     * @brief ڑɌĂ΂R[obNZbg
     *
     * ̃Iy[V́Ã|[g̐ڑɌĂ΂AR[ob
     * Nt@N^ZbgB
     *
     * R[obNt@N^̏ĹAĂяoɂAIuWFNg
     * KvȂȂɉ̂̂͌Ăяo̐ӔCłB
     *
     * ̃R[obNt@N^́A|[g̐ڑs֐ł
     * notify_disconnect() ̏IOɁAĂяoR[obNłB
     * 
     * ̃R[obNt@N^͐ڑɑΉ ConnectorProfile Ƃ
     * ɌĂяoB ConnectorProfile ͂̃t@N^ďo
     * ɔĵŁAύXقɉe^邱Ƃ͂ȂB
     *
     * @param on_disconnected ConnectionCallback ̃TuNXIuWFN
     * g̃|C^
     *
     * @else
     *
     * @brief Setting callback called on disconnected
     *
     * This operation sets a functor that is called when connection
     * between ports is destructed.
     *
     * Since the ownership of the callback functor object is owned by
     * the caller, it has the responsibility of object destruction.
     * 
     * The callback functor is called just before notify_disconnect()
     * that is disconnection execution function returns.
     *
     * This functor is called with argument of corresponding
     * ConnectorProfile.  Since this ConnectorProfile will be
     * destructed after calling this functor, modifications never
     * affect others.
     *
     * @param on_disconnected a pointer to ConnectionCallback's subclasses
     *
     * @endif
     */
    void setOnDisconnected(ConnectionCallback* on_disconnected);

    /*!
     * @if jp
     * @brief |[g̐ڑXgꍇɌĂяoR[obNZbg
     *
     * ̃Iy[V́Ã|[g̐ڑXgꍇɌĂ΂A
     * R[obNt@N^ZbgB
     *
     * InPort́A葤OutPortƂ
     * ڑXgꍇAڑIɐؒf̂ŁA
     *  OnDisconnect R[obNĂяoB
     *
     * @param on_connection_lost ConnectionCallback ̃TuNXIuWFN
     * g̃|C^
     *
     * @else
     * @brief Setting callback called on connection lost
     *
     * This operation sets a functor that is called when connection
     * of this port does lost. 
     *
     * @param on_connection_lost a pointer to ConnectionCallback's subclasses
     *
     * @endif
     */
    void setOnConnectionLost(ConnectionCallback* on_connection_lost);

    /*!
     * @if jp
     * @brief PortConnectListeners ̃z_Zbg
     *
     * |[g̐ڑɊւ郊XiQێz_NXւ̃|C^
     * ZbgB̊֐͒ʏeRTObjectĂ΂ARTObject
     * z_NXւ̃|C^ZbgB
     *
     * @param portconnListeners PortConnectListeners IuWFNg̃|C^
     *
     * @else
     * @brief Setting PortConnectListener holder
     *
     * This operation sets a functor that is called when connection
     * of this port does lost. 
     *
     * @param on_connection_lost a pointer to ConnectionCallback's subclasses
     *
     * @endif
     */
    void setPortConnectListenerHolder(PortConnectListeners* portconnListeners);

	/*!
	* @if jp
	* @brief directʐMp|[gIuWFNg擾
	*
	* @return |[g̃|C^
	*
	* @else
	* @brief 
	*
	* @return
	*
	* @endif
	*/
	virtual DirectPortBase* getDirectPort();
    //============================================================
    // protected operations
    //============================================================
  protected:
    /*! @if jp
     *
     * @brief Interface J
     *
     * ̃Iy[V́Anotify_connect() V[PX̎n߂ɃR[
     * 鏃z֐łB
     * notify_connect() ł́A
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * ̏ protected ֐R[ڑsB
     * <br>
     *  Port ł͂̃Iy[VI[o[ChAƂ
     * ^ꂽ ConnectorProfile ɏ]sAp[^sK
     * ł΁ARteurnCode_t ^̃G[R[hԂB
     * ʏ publishInterafaces() ɂẮA Port ɑ
     * C^[tF[XɊւ ConnectorProfile ɑ΂ēK؂ɐݒ肵
     *  Port ɒʒmȂ΂ȂȂB
     * <br>
     * ܂Å֐R[iKł́A Port  Interface Ɋւ
     * ׂ͂Ċ܂܂ĂȂ̂ŁA Port  Interface 擾鏈
     *  subscribeInterfaces() ōsׂłB
     * <br>
     * ̃Iy[V́AVK connector_id ɑ΂Ă͐ڑ̐A
     *  connector_id ɑ΂Ă͍XVK؂ɍsKvB
     *
     * @param connector_profile ڑɊւvt@C
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Publish interface information
     *
     * This operation is pure virutal function that would be called at the
     * beginning of the notify_connect() process sequence.
     * In the notify_connect(), the following methods would be called in order.
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile() 
     *
     * In the concrete Port, this method should be overridden. This method
     * processes the given ConnectorProfile argument and if the given parameter
     * is invalid, it would return error code of ReturnCode_t.
     * Usually, publishInterfaces() method should set interfaces information
     * owned by this Port, and publish it to the other Ports.
     * <br>
     * When this method is called, other Ports' interfaces information may not
     * be completed. Therefore, the process to obtain other Port's interfaces
     * information should be done in the subscribeInterfaces() method.
     * <br>
     * This operation should create the new connection for the new
     * connector_id, and should update the connection for the existing
     * connection_id.
     *
     * @param connector_profile The connection profile information
     *
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t
    publishInterfaces(ConnectorProfile& connector_profile) = 0;
    
    /*!
     * @if jp
     *
     * @brief  Port ɑ΂ notify_connect() R[
     *
     * ConnectorProfile  port_ref Ɋi[Ă Port ̃IuWFNg
     * t@X̃V[PX̒Ag Port ̎ Port ɑ΂
     * notify_connect() R[B
     *
     * @param connector_profile ڑɊւvt@C
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Call notify_connect() of the next Port
     *
     * This operation calls the notify_connect() of the next Port's 
     * that stored in ConnectorProfile's port_ref sequence.
     *
     * @param connector_profile The connection profile information
     *
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t connectNext(ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     *
     * @brief  Port ɑ΂ notify_disconnect() R[
     *
     * ConnectorProfile  port_ref Ɋi[Ă Port ̃IuWFNg
     * t@X̃V[PX̒Ag Port ̎ Port ɑ΂
     * notify_disconnect() R[B
     *
     * @param connector_profile ڑɊւvt@C
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Call notify_disconnect() of the next Port
     *
     * This operation calls the notify_disconnect() of the next Port's 
     * that stored in ConnectorProfile's port_ref sequence.
     *
     * @param connector_profile The connection profile information
     *
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t disconnectNext(ConnectorProfile& connector_profile);
    
    /*! @if jp
     *
     * @brief Interface J
     *
     * ̃Iy[V́Anotify_connect() V[PX̒ԂɃR[
     * 鏃z֐łB
     * notify_connect() ł́A
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * ̏ protected ֐R[ڑsB
     * <br>
     *  Port ł͂̃Iy[VI[o[ChAƂ
     * ^ꂽ ConnectorProfile ɏ]sAp[^sK
     * ł΁ARteurnCode_t ^̃G[R[hԂB
     *  ConnectorProfile ɂ͑ Port  Interface Ɋւ
     * SĊ܂܂ĂB
     * ʏ subscribeInterafaces() ɂẮA Port gp
     * Interface Ɋւ擾AṽC^[tF[Xɑ΂
     * ݒ肵Ȃ΂ȂȂB
     * <br>
     * ̃Iy[V́AVK connector_id ɑ΂Ă͐ڑ̐A
     *  connector_id ɑ΂Ă͍XVK؂ɍsKvB
     *
     * @param connector_profile ڑɊւvt@C
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Publish interface information
     *
     * This operation is pure virutal function that would be called at the
     * mid-flow of the notify_connect() process sequence.
     * In the notify_connect(), the following methods would be called in order.
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * In the concrete Port, this method should be overridden. This method
     * processes the given ConnectorProfile argument and if the given parameter
     * is invalid, it would return error code of ReturnCode_t.
     * The given argument ConnectorProfile includes all the interfaces
     * information in it.
     * Usually, subscribeInterafaces() method obtains information of interfaces
     * from ConnectorProfile, and should set it to the interfaces that require
     * them.
     * <br>
     * This operation should create the new connection for the new
     * connector_id, and should update the connection for the existing
     * connection_id.
     *
     * @param connector_profile The connection profile information
     *
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t
    subscribeInterfaces(const ConnectorProfile& connector_profile) = 0;
    
    /*! @if jp
     *
     * @brief Interface ̐ڑ
     *
     * ̃Iy[V́Anotify_disconnect() V[PX̏IɃR[
     * 鏃z֐łB
     * notify_disconnect() ł́A
     * - disconnectNext()
     * - unsubscribeInterfaces()
     * - eraseConnectorProfile()
     * ̏ protected ֐R[ڑsB
     * <br>
     *  Port ł͂̃Iy[VI[o[ChAƂ
     * ^ꂽ ConnectorProfile ɏ]ڑsB
     *
     * @param connector_profile ڑɊւvt@C
     *
     * @else
     *
     * @brief Disconnect interface connection
     *
     * This operation is pure virutal function that would be called at the
     * end of the notify_disconnect() process sequence.
     * In the notify_disconnect(), the following methods would be called
     * in order to disconnect.
     * - disconnectNext()
     * - unsubscribeInterfaces()
     * - eraseConnectorProfile() 
     *  <br>
     * In the concrete Port, this method should be overridden. This method
     * processes the given ConnectorProfile argument and disconnect interface
     * connection.
     *
     * @param connector_profile The connection profile information
     *
     * @endif
     */
    virtual void
    unsubscribeInterfaces(const ConnectorProfile& connector_profile) = 0;

    /*! @if jp
     *
     * @brief ڑ̍ő吔ݒ肷B
     *
     * @param limit_value ő吔
     *
     * @else
     *
     * @brief Set the maximum number of connections
     *
     *
     * @param limit_value The maximum number of connections
     *
     * @endif
     */
    virtual void setConnectionLimit(int limit_value);
    
    /*!
     * @if jp
     * @brief InterfaceJ
     *
     * InterfaceJB
     *
     *  dataport.dataflow_type
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     * @brief Publish interface information
     *
     * Publish interface information.
     *
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t _publishInterfaces(void);
    //============================================================
    // protected utility functions
    //============================================================
    /*!
     * @if jp
     *
     * @brief ConnectorProfile  connector_id tB[h󂩂ǂ
     *
     * w肳ꂽ ConnectorProfile  connector_id ł邩ǂ̔
     * sB
     *
     * @param connector_profile ΏۃRlN^vt@C
     *
     * @return ŗ^ꂽ ConnectorProfile  connector_id ł΁A
     *         trueAłȂ false ԂB
     *
     * @else
     *
     * @brief Check whether connector_id of ConnectorProfile is empty
     *
     * Check whether connector_id of specified ConnectorProfile is empty.
     * 
     * @param connector_profile Target ConnectorProfile for the check
     *
     * @return If the given ConnectorProfile's connector_id is empty string,
     *         it returns true.
     *
     * @endif
     */
    bool isEmptyId(const ConnectorProfile& connector_profile) const;
    
    /*!
     * @if jp
     *
     * @brief UUID𐶐
     *
     * ̃Iy[V UUID 𐶐B
     *
     * @return uuid
     *
     * @else
     *
     * @brief Generate the UUID
     *
     * This operation generates UUID.
     *
     * @return uuid
     *
     * @endif
     */
    const std::string getUUID() const;
    
    /*!
     * @if jp
     *
     * @brief UUID𐶐 ConnectorProfile ɃZbg
     *
     * ̃Iy[V UUID 𐶐AConnectorProfile ɃZbgB
     *
     * @param connector_profile connector_id Zbg ConnectorProfile
     *
     * @else
     *
     * @brief Generate and set the UUID to the ConnectorProfile
     *
     * This operation generates and set UUID to the ConnectorProfile.
     *
     * @param connector_profile ConnectorProfile to be set connector_id
     *
     * @endif
     */
    void setUUID(ConnectorProfile& connector_profile) const;
    
    /*!
     * @if jp
     *
     * @brief id  ConnectorProfile ̂̂ǂ肷
     *
     * ̃Iy[V͗^ꂽ ID  ConnectorProfile ̃Xg
     * ݂邩ǂ肷B
     *
     * @param id 肷 connector_id
     *
     * @return id ̑ݔ茋
     *
     * @else
     *
     * @brief Check whether the given id exists in stored ConnectorProfiles
     *
     * This operation returns boolean whether the given id exists in 
     * the Port's ConnectorProfiles.
     *
     * @param id connector_id to be find in Port's ConnectorProfiles
     *
     * @return id exestance resutl
     *
     * @endif
     */
    bool isExistingConnId(const char* id);
    
    /*!
     * @if jp
     *
     * @brief id  ConnectorProfile T
     *
     * ̃Iy[V͗^ꂽ ID  ConnectorProfile  Port 
     *  ConnectorProfile ̃XgTB
     * A id  ConnectorProfile Ȃ΁A ConnectorProfile
     * ԂB
     *
     * @param id  connector_id
     *
     * @return connector_id  ConnectorProfile
     *
     * @else
     *
     * @brief Find ConnectorProfile with id
     *
     * This operation returns ConnectorProfile with the given id from Port's
     * ConnectorProfiles' list.
     * If the ConnectorProfile with connector id that is identical with the
     * given id does not exist, empty ConnectorProfile is returned.
     *
     * @param id the connector_id to be searched in Port's ConnectorProfiles
     *
     * @return CoonectorProfile with connector_id
     *
     * @endif
     */
    ConnectorProfile findConnProfile(const char* id);
    
    /*!
     * @if jp
     *
     * @brief id  ConnectorProfile T
     *
     * ̃Iy[V͗^ꂽ ID  ConnectorProfile  Port 
     *  ConnectorProfile ̃XgTCfbNXԂB
     * A id  ConnectorProfile Ȃ΁A-1 ԂB
     *
     * @param id  connector_id
     *
     * @return Port  ConnectorProfile Xg̃CfbNX
     *
     * @else
     *
     * @brief Find ConnectorProfile with id
     *
     * This operation returns ConnectorProfile with the given id from Port's
     * ConnectorProfiles' list.
     * If the ConnectorProfile with connector id that is identical with the
     * given id does not exist, -1 is returned.
     *
     * @param id the connector_id to be searched
     *
     * @return The index of ConnectorProfile of the Port
     *
     * @endif
     */
    CORBA::Long findConnProfileIndex(const char* id);
    
    /*!
     * @if jp
     *
     * @brief ConnectorProfile ̒ǉ͍XV
     *
     * ̃Iy[V͗^ꂽ ConnectorProfile  Port ɒǉ
     * XVۑB
     * ^ꂽ ConnectorProfile  connector_id Ɠ ID 
     * ConnectorProfile XgɂȂ΁AXgɒǉA
     *  ID ݂ ConnectorProfile ㏑ۑB
     *
     * @param connector_profile ǉ͍XV ConnectorProfile
     *
     * @else
     *
     * @brief Append or update the ConnectorProfile list
     *
     * This operation appends or updates ConnectorProfile of the Port
     * by the given ConnectorProfile.
     * If the connector_id of the given ConnectorProfile does not exist
     * in the Port's ConnectorProfile list, the given ConnectorProfile would be
     * append to the list. If the same id exists, the list would be updated.
     *
     * @param connector_profile the ConnectorProfile to be appended or updated
     *
     * @endif
     */
    void updateConnectorProfile(const ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     *
     * @brief ConnectorProfile 폜
     *
     * ̃Iy[V Port  PortProfile ێĂ
     * ConnectorProfileList ̂^ꂽ id  ConnectorProfile
     * 폜B
     *
     * @param id 폜 ConnectorProfile  id
     *
     * @return ɍ폜łꍇ trueA
     *         w肵 ConnectorProfile Ȃꍇ false Ԃ
     *
     * @else
     *
     * @brief Delete the ConnectorProfile
     *
     * This operation deletes a ConnectorProfile specified by id from
     * ConnectorProfileList owned by PortProfile of this Port.
     *
     * @param id The id of the ConnectorProfile to be deleted.
     *
     * @return true would be returned if it deleted correctly.
     *         false woluld be returned if specified ConnectorProfile
     *         cannot be found.
     *
     * @endif
     */
    bool eraseConnectorProfile(const char* id);
    
    /*!
     * @if jp
     *
     * @brief PortInterfaceProfile  C^[tF[Xo^
     *
     * ̃Iy[V Port  PortProfile ́APortInterfaceProfile
     * ɃC^[tF[X̏ǉB
     * ̏́Aget_port_profile() ɂē PortProfile ̂
     * PortInterfaceProfile ̒lύX݂̂łAۂɃC^[tF[X
     * 񋟂v肷ꍇɂ́ATuNXŁA publishInterface() ,
     *  subscribeInterface() ̊֐K؂ɃI[o[ChC^[tF[X
     * 񋟁AvsȂ΂ȂȂB
     *
     * C^[tF[X(̃CX^X) Port ňӂłȂ΂ȂȂB
     * ̃C^[tF[Xłɓo^ĂꍇÅ֐ false 
     * ԂB
     *
     * @param name C^[tF[X̃CX^X̖O
     * @param type_name C^[tF[X̌^̖O
     * @param pol C^[tF[X̑ (RTC::PROVIDED  RTC:REQUIRED)
     *
     * @return C^[tF[Xo^ʁB
     *         ̃C^[tF[Xɓo^Ă false ԂB
     *
     * @else
     *
     * @brief Append an interface to the PortInterfaceProfile
     *
     * This operation appends interface information to the PortInterfaceProfile
     * that is owned by the Port.
     * The given interfaces information only updates PortInterfaceProfile of
     * PortProfile that is obtained through get_port_profile().
     * In order to provide and require interfaces, proper functions (for
     * example publishInterface(), subscribeInterface() and so on) should be
     * overridden in subclasses, and these functions provide concrete interface
     * connection and disconnection functionality.
     *
     * The interface (instance) name have to be unique in the Port.
     * If the given interface name is identical with stored interface name,
     * this function returns false.
     *
     * @param name The instance name of the interface.
     * @param type_name The type name of the interface.
     * @param pol The interface's polarity (RTC::PROVIDED or RTC:REQUIRED)
     *
     * @return false would be returned if the same name is already registered.
     *
     * @endif
     */
    bool appendInterface(const char* name, const char* type_name,
			 PortInterfacePolarity pol);
    
    /*!
     * @if jp
     *
     * @brief PortInterfaceProfile C^[tF[Xo^폜
     *
     * ̃Iy[V Port  PortProfile ́APortInterfaceProfile
     * C^[tF[X̏폜B
     *
     * @param name C^[tF[X̃CX^X̖O
     * @param pol C^[tF[X̑ (RTC::PROVIDED  RTC:REQUIRED)
     *
     * @return C^[tF[X폜ʁB
     *         C^[tF[Xo^ĂȂ false ԂB
     *
     * @else
     *
     * @brief Delete the interface registration from the PortInterfaceProfile
     *
     * This operation deletes interface information from the
     * PortInterfaceProfile that is owned by the Port.
     *
     * @param name The instance name of the interface.
     * @param pol The interface's polarity (RTC::PROVIDED or RTC:REQUIRED)
     *
     * @return Delete processing result of interface.
     *         false would be returned if the given name is not registered.
     *
     * @endif
     */
    bool deleteInterface(const char* name, PortInterfacePolarity pol);
    
    /*!
     * @if jp
     *
     * @brief PortProfile  properties  NameValue lǉ
     *
     * PortProfile  properties  NameValue lǉB
     * ǉf[^̌^ValueTypeŎw肷B
     *
     * @param key properties  name
     * @param value properties  value
     *
     * @else
     *
     * @brief Add NameValue data to PortProfile's properties
     *
     * Add NameValue data to PortProfile's properties.
     * Type of additional data is specified by ValueType.
     *
     * @param key The name of properties
     * @param value The value of properties
     *
     * @endif
     */
    template <class ValueType>
    void addProperty(const char* key, ValueType value)
    {
      CORBA_SeqUtil::push_back(m_profile.properties,
			       NVUtil::newNV(key, value));
    }

    /*!
     * @if jp
     *
     * @brief PortProfile  properties  NameValue lvfɒǉ
     *
     * PortProfile  properties  NameValue lvfɒǉB
     * ֐ɂݒ肳ꂽ properties  get_prot_profile() ɂO
     * QƂB
     *
     * @param key properties  name
     * @param value properties  value
     *
     * @else
     *
     * @brief Append NameValue data to PortProfile's properties
     *
     * Append NameValue data to PortProfile's properties.  The
     * properties which are set by this function would be referred
     * through get_port_profile() from outsides.
     *
     * @param key The name of properties
     * @param value The value of properties
     *
     * @endif
     */
    void appendProperty(const char* key, const char* value)
    {
      NVUtil::appendStringValue(m_profile.properties, key, value);
    }
    /*!
     * @if jp
     * @brief vpeB
     * @else
     * @brief Properties
     * @endif
     */
    coil::Properties m_properties;
  protected:
    /*!
     * @if jp
     *
     * @brief ݂Ȃ|[gdisconnectB
     *
     * PortoAł|[gꍇɂ́Aڑ
     * B
     *
     * @else
     *
     * @brief Disconnect ports that doesn't exist. 
     *
     * This function detects dead-port, and if dead ports are found in
     * the connection list, disconnects them.
     *
     * @endif
     */
    void updateConnectors();

    /*!
     * @if jp
     *
     * @brief |[g݂̑mFB
     *
     * @param ports mF|[g
     * @return true:݂,false:݂Ȃ
     *
     * @else
     *
     * @brief Existence of ports
     *
     * @param ports Checked ports
     * @return true:existent,false:non existent
     *
     * @endif
     */
#ifndef ORB_IS_RTORB
    bool checkPorts(::RTC::PortServiceList& ports);
#else // ORB_IS_RTORB
    bool checkPorts(RTC_PortServiceList& ports);
#endif // ORB_IS_RTORB


    inline void onNotifyConnect(const char* portname,
                                RTC::ConnectorProfile& profile)
    {
      if (m_portconnListeners != NULL)
        {
          m_portconnListeners->
            portconnect_[ON_NOTIFY_CONNECT].notify(portname, profile);
        }
    }

    inline void onNotifyDisconnect(const char* portname,
                                   RTC::ConnectorProfile& profile)
    {
      if (m_portconnListeners != NULL)
        {
          m_portconnListeners->
            portconnect_[ON_NOTIFY_DISCONNECT].notify(portname, profile);
        }
    }
    inline void onUnsubscribeInterfaces(const char* portname,
                                        RTC::ConnectorProfile& profile)
    {
      if (m_portconnListeners != NULL)
        {
          m_portconnListeners->
            portconnect_[ON_UNSUBSCRIBE_INTERFACES].notify(portname, profile);
        }
    }

    inline void onPublishInterfaces(const char* portname,
                                    RTC::ConnectorProfile& profile,
                                    ReturnCode_t ret)
    {
      if (m_portconnListeners != NULL)
        {
          m_portconnListeners->
            portconnret_[ON_PUBLISH_INTERFACES].notify(portname,
                                                       profile, ret);
        }
    }

    inline void onConnectNextport(const char* portname,
                                  RTC::ConnectorProfile& profile,
                                  ReturnCode_t ret)
    {
      if (m_portconnListeners != NULL)
        {
          m_portconnListeners->
            portconnret_[ON_CONNECT_NEXTPORT].notify(portname,
                                                     profile, ret);
        }
    }

    inline void onSubscribeInterfaces(const char* portname,
                                      RTC::ConnectorProfile& profile,
                                      ReturnCode_t ret)
    {
      if (m_portconnListeners != NULL)
        {
          m_portconnListeners->
            portconnret_[ON_SUBSCRIBE_INTERFACES].notify(portname,
                                                         profile, ret);
        }
    }

    inline void onConnected(const char* portname,
                            RTC::ConnectorProfile& profile,
                            ReturnCode_t ret)
    {
      if (m_portconnListeners != NULL)
        {
          m_portconnListeners->
            portconnret_[ON_CONNECTED].notify(portname, profile, ret);
        }
    }

    inline void onDisconnectNextport(const char* portname,
                                 RTC::ConnectorProfile& profile,
                                 ReturnCode_t ret)
    {
      if (m_portconnListeners != NULL)
        {
          m_portconnListeners->
            portconnret_[ON_DISCONNECT_NEXT].notify(portname, profile, ret);
        }
    }

    inline void onDisconnected(const char* portname,
                               RTC::ConnectorProfile& profile,
                               ReturnCode_t ret)
    {
      if (m_portconnListeners != NULL)
        {
          m_portconnListeners->
            portconnret_[ON_DISCONNECTED].notify(portname, profile, ret);
        }
    }

  protected:
    /*!
     * @if jp
     * @brief K[Xg[
     * @else
     * @brief Logger stream
     * @endif
     */
    mutable Logger rtclog;
    /*!
     * @if jp
     * @brief Port  PortProfile
     * @else
     * @brief PortProfile of the Port
     * @endif
     */
    PortProfile m_profile;
    
    /*!
     * @if jp
     * @brief Port  IuWFNgQ
     * @else
     * @brief Object Reference of the Port
     * @endif
     */
    RTC::PortService_var m_objref;
    /*!
     * @if jp
     * @brief PortProfile  mutex
     * @else
     * @brief Mutex of PortProfile
     * @endif
     */
    mutable coil::Mutex m_profile_mutex;
    mutable coil::Mutex m_connectorsMutex;
    typedef coil::Guard<coil::Mutex> Guard;

    /*!
     * @if jp
     * @brief CX^X
     * @else
     * @brief Instance name
     * @endif
     */
    std::string m_ownerInstanceName;

    /*!
     * @if jp
     * @brief Port ̐ڑ̍ő吔
     * @else
     * @brief The maximum number of connections
     * @endif
     */
    int m_connectionLimit;
    
    /*!
     * @if jp
     * @brief Callback functor IuWFNg
     *
     * C^[tF[XJۂɌĂ΂R[obNIuWFNg
     *
     * @else
     * @brief Callback functor objects
     *
     * This is callback objedct that is called when the interface is opened 
     * to the public.
     *
     * @endif
     */
    ConnectionCallback* m_onPublishInterfaces;
    /*!
     * @if jp
     * @brief Callback functor IuWFNg
     *
     * C^[tF[X擾ۂɌĂ΂R[obNIuWFNg
     *
     * @else
     * @brief Callback functor objects
     *
     * This is callback objedct that is called when the interface is got.
     *
     * @endif
     */
    ConnectionCallback* m_onSubscribeInterfaces;
    /*!
     * @if jp
     * @brief Callback functor IuWFNg
     *
     * ڑɌĂ΂R[obNIuWFNg
     *
     * @else
     * @brief Callback functor objects
     *
     * This is a callback object that is called 
     * when the connection is completed. 
     *
     * @endif
     */
    ConnectionCallback* m_onConnected;
    /*!
     * @if jp
     * @brief Callback functor IuWFNg
     *
     * C^[tF[XۂɌĂ΂R[obNIuWFNg
     *
     * @else
     * @brief Callback functor objects
     *
     * This is a callback object that is called when the interface is released. 
     *
     * @endif
     */
    ConnectionCallback* m_onUnsubscribeInterfaces;
    /*!
     * @if jp
     * @brief Callback functor IuWFNg
     *
     * ڑɌĂ΂R[obNIuWFNg
     *
     * @else
     * @brief Callback functor objects
     *
     * This is a callback object that is called in connected release. 
     *
     * @endif
     */
    ConnectionCallback* m_onDisconnected;

    /*!
     * @if jp
     * @brief Callback functor IuWFNg
     *
     * |[g̐ڑXgꍇɌĂяoR[obNIuWFNg
     *
     * @else
     * @brief Callback functor objects
     *
     * This is a callback object called when the connection of the port does 
     * lost. 
     *
     * @endif
     */
    ConnectionCallback* m_onConnectionLost;

    /*!
     * @if jp
     * @brief PortConnectListenerz_
     *
     * PortConnectListenrێz_
     *
     * @else
     * @brief PortConnectListener holder
     *
     * Holders of PortConnectListeners
     *
     * @endif
     */
    PortConnectListeners* m_portconnListeners;

	DirectPortBase *m_directport;

    //============================================================
    // Functor
    //============================================================
    /*!
     * @if jp
     * @brief id  ConnectorProfile T Functor
     * @else
     * @brief Functor to find a ConnectorProfile named id
     * @endif
     */
    struct find_conn_id
    {
      find_conn_id(const char* id) : m_id(id) {};
      bool operator()(const ConnectorProfile& cprof)
      {
	return m_id == std::string(cprof.connector_id);
      }
      std::string m_id;
    };  // struct find_conn_id
    
    /*!
     * @if jp
     * @brief RXgN^ port_ref ƓIuWFNgQƂT Functor
     * @else
     * @brief Functor to find the object reference that is identical port_ref
     * @endif
     */
    struct find_port_ref
    {
      find_port_ref(PortService_ptr port_ref) : m_port(port_ref) {};
      bool operator()(PortService_ptr port_ref)
      {
	return m_port->_is_equivalent(port_ref);
      }
      PortService_ptr m_port;
    };  // struct find_port_ref
    
    /*!
     * @if jp
     * @brief name  polarity  interface T Functor
     * @else
     * @brief Functor to find interface from name and polarity
     * @endif
     */
    struct find_interface
    {
      find_interface(const char* name, PortInterfacePolarity pol)
	: m_name(name), m_pol(pol)
      {}
      
      bool operator()(const PortInterfaceProfile& prof)
      {
	CORBA::String_var name(CORBA::string_dup(prof.instance_name));
	return ((m_name == (const char *)name) && (m_pol == prof.polarity));
      }
      std::string m_name;
      PortInterfacePolarity m_pol;
    };  // struct find_interface
  };  // class PortBase
};  // namespace RTC

#ifdef WIN32
#pragma warning( default : 4290 )
#endif

#endif // RTC_PORTBASE_H
