// -*- C++ -*-
/*!
 * @file PortConnectListener.h
 * @brief port's internal action listener classes
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2011
 *     Noriaki Ando
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTC_PORTCONNECTLISTENER_H
#define RTC_PORTCONNECTLISTENER_H

#include <vector>
#include <utility>
#include <coil/Mutex.h>
#include <coil/Guard.h>
#include <rtm/RTC.h>
#include <rtm/idl/RTCSkel.h>

namespace RTC
{
  //============================================================
  /*!
   * @if jp
   * @brief PortConnectListener ̃^Cv
   *
   * - ON_NOTIFY_CONNECT:         notify_connect() ֐Ăяo
   * - ON_NOTIFY_DISCONNECT:      notify_disconnect() Ăяo
   * - ON_UNSUBSCRIBE_INTERFACES: notify_disconnect() IFwǉ
   *
   * @else
   * @brief The types of ConnectorDataListener
   * 
   * - ON_NOTIFY_CONNECT:         right after entering into notify_connect()
   * - ON_NOTIFY_DISCONNECT:      right after entering into notify_disconnect()
   * - ON_UNSUBSCRIBE_INTERFACES: unsubscribing IF in notify_disconnect()
   *
   * @endif
   */
  enum PortConnectListenerType
    {
      ON_NOTIFY_CONNECT,
      ON_NOTIFY_DISCONNECT,
      ON_UNSUBSCRIBE_INTERFACES,
      PORT_CONNECT_LISTENER_NUM
    };

  /*!
   * @if jp
   * @class PortConnectListener NX
   * @brief PortConnectListener NX
   *
   * eANVɑΉ郆[U[R[hĂ΂钼Õ^C~O
   * ŃR[郊XiNX̊NXB
   *
   * - ON_NOTIFY_CONNECT:         notify_connect() ֐Ăяo
   * - ON_NOTIFY_DISCONNECT:      notify_disconnect() Ăяo
   * - ON_UNSUBSCRIBE_INTERFACES: notify_disconnect() IFwǉ
   *
   * @else
   * @class PortConnectListener class
   * @brief PortConnectListener class
   *
   * This class is abstract base class for listener classes that
   * provides callbacks for various events in rtobject.
   *
   * - ON_NOTIFY_CONNECT:         right after entering into notify_connect()
   * - ON_NOTIFY_DISCONNECT:      right after entering into notify_disconnect()
   * - ON_UNSUBSCRIBE_INTERFACES: unsubscribing IF in notify_disconnect()
   *
   * @endif
   */
  class PortConnectListener
  {
  public:
    /*!
     * @if jp
     *
     * @brief PortConnectListenerType 𕶎ɕϊ
     *
     * PortConnectListenerType 𕶎ɕϊ
     *
     * @param type ϊΏ PortConnectListenerType
     *
     * @return ϊ
     *
     * @else
     *
     * @brief Convert PortConnectListenerType into the string.
     *
     * Convert PortConnectListenerType into the string.
     *
     * @param type The target PortConnectListenerType for transformation
     *
     * @return Trnasformation result of string representation
     *
     * @endif
     */
    static const char* toString(PortConnectListenerType type);

    /*!
     * @if jp
     * @brief fXgN^
     * @else
     * @brief Destructor
     * @endif
     */
    virtual ~PortConnectListener();

    /*!
     * @if jp
     *
     * @brief zR[obN֐
     *
     * PortConnectListener ̃R[obN֐
     *
     * @else
     *
     * @brief Virtual Callback function
     *
     * This is a the Callback function for PortConnectListener.
     *
     * @endif
     */
    virtual void operator()(const char* portname,
                            RTC::ConnectorProfile& profile) = 0;
  };


  //============================================================
  /*!
   * @if jp
   * @brief PortConnectRetListenerType ̃^Cv
   *
   * - ON_CONNECT_NEXTPORT:     notify_connect() ̃JXP[hĂяo
   * - ON_SUBSCRIBE_INTERFACES: notify_connect() ̃C^[tF[Xwǒ
   * - ON_CONNECTED:            nofity_connect() ڑɌĂяo
   * - ON_DISCONNECT_NEXT:      notify_disconnect() ɃJXP[hĂяo
   * - ON_DISCONNECTED:         notify_disconnect() ^[
   *
   * @else
   * @brief The types of PortConnectRetListenerType
   * 
   * - ON_CONNECT_NEXTPORT:     after cascade-call in notify_connect()
   * - ON_SUBSCRIBE_INTERFACES: after IF subscribing in notify_connect()
   * - ON_CONNECTED:            completed nofity_connect() connection process
   * - ON_DISCONNECT_NEXT:      after cascade-call in notify_disconnect()
   * - ON_DISCONNECTED:         completed notify_disconnect() disconnection
   *
   * @endif
   */
  enum PortConnectRetListenerType
    {
      ON_PUBLISH_INTERFACES,
      ON_CONNECT_NEXTPORT,
      ON_SUBSCRIBE_INTERFACES,
      ON_CONNECTED,
      ON_DISCONNECT_NEXT,
      ON_DISCONNECTED,
      PORT_CONNECT_RET_LISTENER_NUM
    };

   /*!
   * @if jp
   * @class PortConnectRetListener NX
   * @brief PortConnectRetListener NX
   *
   * eANVɑΉ郆[U[R[hĂ΂钼Õ^C~O
   * ŃR[郊XȃNX̊NXB
   *
   * - ON_PUBLISH_INTERFACES:   notify_connect() ̃C^[tF[XJ
   * - ON_CONNECT_NEXTPORT:     notify_connect() ̃JXP[hĂяo
   * - ON_SUBSCRIBE_INTERFACES: notify_connect() ̃C^[tF[Xwǒ
   * - ON_CONNECTED:            nofity_connect() ڑɌĂяo
   * - ON_DISCONNECT_NEXT:      notify_disconnect() ɃJXP[hĂяo
   * - ON_DISCONNECTED:         notify_disconnect() ^[
   *
   * @else
   * @class PortConnectRetListener class
   * @brief PortConnectRetListener class
   *
   * This class is abstract base class for listener classes that
   * provides callbacks for various events in rtobject.
   *
   * - ON_CONNECT_NEXTPORT:     after cascade-call in notify_connect()
   * - ON_SUBSCRIBE_INTERFACES: after IF subscribing in notify_connect()
   * - ON_CONNECTED:            completed nofity_connect() connection process
   * - ON_DISCONNECT_NEXT:      after cascade-call in notify_disconnect()
   * - ON_DISCONNECTED:         completed notify_disconnect() disconnection
   *
   * @endif
   */
  class PortConnectRetListener
  {
  public:
    /*!
     * @if jp
     *
     * @brief PortConnectRetListenerType 𕶎ɕϊ
     *
     * PortConnectRetListenerType 𕶎ɕϊ
     *
     * @param type ϊΏ PortConnectRetListenerType
     *
     * @return ϊ
     *
     * @else
     *
     * @brief Convert PortConnectRetListenerType into string.
     *
     * Convert PortConnectRetListenerType into string.
     *
     * @param type The target PortConnectRetListenerType for transformation
     *
     * @return Trnasformation result of string representation
     *
     * @endif
     */
    static const char* toString(PortConnectRetListenerType type);

    /*!
     * @if jp
     * @brief fXgN^
     * @else
     * @brief Destructor
     * @endif
     */
    virtual ~PortConnectRetListener();

    /*!
     * @if jp
     *
     * @brief zR[obN֐
     *
     * PortConnectRetListener ̃R[obN֐
     *
     * @else
     *
     * @brief Virtual Callback function
     *
     * This is a the Callback function for PortConnectRetListener.
     *
     * @endif
     */
    virtual void operator()(const char* portname,
                            RTC::ConnectorProfile& profile,
                            ReturnCode_t ret) = 0;
  };

  //============================================================
  /*!
   * @if jp
   * @class PortConnectListenerHolder 
   * @brief PortConnectListener z_NX
   *
   *  PortConnectListener ێǗNXB
   *
   * @else
   * @class PortConnectListenerHolder
   * @brief PortConnectListener holder class
   *
   * This class manages one ore more instances of
   * PortConnectListener class.
   *
   * @endif
   */
  class PortConnectListenerHolder
  {
    typedef std::pair<PortConnectListener*, bool> Entry;
    typedef coil::Guard<coil::Mutex> Guard;
  public:
    /*!
     * @if jp
     * @brief RXgN^
     * @else
     * @brief Constructor
     * @endif
     */
    PortConnectListenerHolder();
    
    /*!
     * @if jp
     * @brief fXgN^
     * @else
     * @brief Destructor
     * @endif
     */
    virtual ~PortConnectListenerHolder();
    
    /*!
     * @if jp
     *
     * @brief Xi[̒ǉ
     *
     * Xi[ǉB
     *
     * @param listener ǉ郊Xi
     * @param autoclean true:fXgN^ō폜,
     *                  false:fXgN^ō폜Ȃ
     * @else
     *
     * @brief Add the listener.
     *
     * This method adds the listener. 
     *
     * @param listener Added listener
     * @param autoclean true:The listener is deleted at the destructor.,
     *                  false:The listener is not deleted at the destructor. 
     * @endif
     */
    void addListener(PortConnectListener* listener, bool autoclean);
    
    /*!
     * @if jp
     *
     * @brief Xi[̍폜
     *
     * Xi폜B
     *
     * @param listener 폜郊Xi
     * @else
     *
     * @brief Remove the listener. 
     *
     * This method removes the listener. 
     *
     * @param listener Removed listener
     * @endif
     */
    void removeListener(PortConnectListener* listener);

    /*!
     * @if jp
     *
     * @brief Xi[֒ʒm
     *
     * o^Ă郊XĩR[obN\bhĂяoB
     *
     * @param info ConnectorInfo
     * @else
     *
     * @brief Notify listeners. 
     *
     * This calls the Callback method of the registered listener. 
     *
     * @param info ConnectorInfo
     * @endif
     */
    void notify(const char* portname, RTC::ConnectorProfile& profile);
      
  private:
    std::vector<Entry> m_listeners;
    coil::Mutex m_mutex;
  };


  /*!
   * @if jp
   * @class PortConnectRetListenerHolder
   * @brief PortConnectRetListener z_NX
   *
   *  PortConnectRetListener ێǗNXB
   *
   * @else
   * @class PortConnectRetListenerHolder
   * @brief PortConnectRetListener holder class
   *
   * This class manages one ore more instances of
   * PortConnectRetListener class.
   *
   * @endif
   */
  class PortConnectRetListenerHolder
  {
    typedef std::pair<PortConnectRetListener*, bool> Entry;
    typedef coil::Guard<coil::Mutex> Guard;
  public:
    /*!
     * @if jp
     * @brief RXgN^
     * @else
     * @brief Constructor
     * @endif
     */
    PortConnectRetListenerHolder();

    /*!
     * @if jp
     * @brief fXgN^
     * @else
     * @brief Destructor
     * @endif
     */
    virtual ~PortConnectRetListenerHolder();
    
    /*!
     * @if jp
     *
     * @brief Xi[̒ǉ
     *
     * Xi[ǉB
     *
     * @param listener ǉ郊Xi
     * @param autoclean true:fXgN^ō폜,
     *                  false:fXgN^ō폜Ȃ
     * @else
     *
     * @brief Add the listener.
     *
     * This method adds the listener. 
     *
     * @param listener Added listener
     * @param autoclean true:The listener is deleted at the destructor.,
     *                  false:The listener is not deleted at the destructor. 
     * @endif
     */
    void addListener(PortConnectRetListener* listener, bool autoclean);
    
    /*!
     * @if jp
     *
     * @brief Xi[̍폜
     *
     * Xi폜B
     *
     * @param listener 폜郊Xi
     * @else
     *
     * @brief Remove the listener. 
     *
     * This method removes the listener. 
     *
     * @param listener Removed listener
     * @endif
     */
    void removeListener(PortConnectRetListener* listener);
    
    /*!
     * @if jp
     *
     * @brief Xi[֒ʒm
     *
     * o^Ă郊XĩR[obN\bhĂяoB
     *
     * @param info ConnectorInfo
     * @param cdrdata f[^
     * @else
     *
     * @brief Notify listeners. 
     *
     * This calls the Callback method of the registered listener. 
     *
     * @param info ConnectorInfo
     * @param cdrdata Data
     * @endif
     */
    void notify(const char* portname, RTC::ConnectorProfile& profile,
                ReturnCode_t ret);
    
  private:
    std::vector<Entry> m_listeners;
    coil::Mutex m_mutex;
  };

  /*!
   * @if jp
   * @class PortConnectListeners
   * @brief PortConnectListeners NX
   *
   *
   * @else
   * @class PortConnectListeners
   * @brief PortConnectListeners class
   *
   *
   * @endif
   */
  class PortConnectListeners
  {
  public:
    /*!
     * @if jp
     * @brief PortConnectListenerType Xiz
     * PortConnectListenerType Xii[
     * @else
     * @brief PortConnectListenerType listener array
     * The PortConnectListenerType listener is stored. 
     * @endif
     */
    PortConnectListenerHolder 
    portconnect_[PORT_CONNECT_LISTENER_NUM];
    /*!
     * @if jp
     * @brief PortConnectRetTypeXiz
     * PortConnectRetTypeXii[
     * @else
     * @brief PortConnectRetType listener array
     * The PortConnectRetType listener is stored.
     * @endif
     */
    PortConnectRetListenerHolder 
    portconnret_[PORT_CONNECT_RET_LISTENER_NUM];
  };


}; // namespace RTC

#endif // RTC_PORTCONNECTLISTENER_H
